c@***h* CMORPH-CDR/a3.CMORPH_V1.0_CRT.get_ADJ_0.25deg-HLY.f
c
c=======================================================================
c PROGRAM
c   a3.CMORPH_V1.0_CRT.get_ADJ_0.25deg-HLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-HLY bias-corrected CMORPH data from 0.25deg-30min
c   bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c USAGE
c   a3.CMORPH_V1.0_CRT.get_ADJ_0.25deg-HLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-30min_<yyyymmdd> : 0.25deg-30min bias-corrected
c                                              CMORPH data daily file
c
c OUTPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-HLY_<yyyymmddhh> : 0.25deg-HLY bias-corrected
c                                              CMORPH data hourly files
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   -----------:--------------------------------------
c   yyyymmdd   : target date
c   f_30m      : 0.25deg-30min bias-corrected CMORPH data file name
c   f_hly      : 0.25deg-HLY bias-corrected CMORPH data file name
c   yyyy,mm,dd : target date
c   hh         : loop counter for hours
c   hf         : loop counter for half-hour
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   ct()       : count of valid data points
c   rain()     : 0.25deg-30min bias-corrected CMORPH data
c   ss()       : 0.25deg-HLY bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_def_0.25deg_HLY.lnx.f'
c
c   step 3 of 6 to generate CMORPH_V1.0_CRT dataset
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program a3_CMORPH_V10_CRT_get_ADJ_025deg_HLY
      implicit none

      integer NX, NY
      parameter( NX=1440, NY=480 )

      character yyyymmdd*8, f_30m*46, f_hly*46
      integer yyyy, mm, dd, hh, hf, ii, jj, ct(NX,NY), tt
      real rain(NX,NY), ss(NX,NY)

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|123456<--
      f_30m='../work/CMORPH_V1.0_ADJ_0.25deg-30min_yyyymmdd'
      f_hly='../work/CMORPH_V1.0_ADJ_0.25deg-HLY_yyyymmddhh'

      if( iargc().ge.2 ) then
        f_30m(4:7)='redo'
        f_hly(4:7)='redo'
      endif

      write(f_30m(39:46), '(i4.4,2i2.2)') yyyy, mm, dd
      
c-----------------------------------------------------------------------
c open 0.25deg-30min bias-corrected CMORPH data daily file for input
c-----------------------------------------------------------------------
      open(10, file=f_30m, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_30m

c-----------------------------------------------------------------------
c generate 0.25deg-HLY bias-corrected CMORPH data in each hour
c-----------------------------------------------------------------------
      do hh=1, 24

        write(f_hly(37:46), '(i4.4,3i2.2)') yyyy, mm, dd, hh-1

        open(20, file=f_hly, access='direct', recl=NX*NY*4)

        do jj=1, NY
          do ii=1, NX
            ct(ii,jj)=0
            ss(ii,jj)=0.0
          enddo
        enddo

        do hf=1, 2
          read(10, rec=(hh-1)*2+hf) rain
          do jj=1, NY
            do ii=1, NX
              if( rain(ii,jj).ge.0.0 ) then
                ct(ii,jj)=ct(ii,jj)+1
                ss(ii,jj)=ss(ii,jj)+rain(ii,jj)
              endif
            enddo
          enddo
        enddo

        do jj=1, NY
          do ii=1, NX
            if( ct(ii,jj).gt.0 ) then
              ss(ii,jj)=ss(ii,jj)/ct(ii,jj)
            else
              ss(ii,jj)=-999.0
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c truncate to 2 digits after decimal point
c-----------------------------------------------------------------------
        do jj=1, NY
          do ii=1, NX
            if( ss(ii,jj).le.-999.0 ) then
              ss(ii,jj)=-999.0
            else
              tt=nint(ss(ii,jj)*100.0)
              ss(ii,jj)=0.01*tt
            endif
          enddo
        enddo

        write(20, rec=1) ss
        close(20)
        write(*, *) '  ==> ', f_hly

      enddo

      close(10)

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
