c@***h* CMORPH-CDR/a4.CMORPH_V1.0_CRT.get_ADJ_0.25deg-3HLY.f
c
c=======================================================================
c PROGRAM
c   a4.CMORPH_V1.0_CRT.get_ADJ_0.25deg-3HLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-3HLY bias-corrected CMORPH data from 0.25deg-HLY
c   bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c USAGE
c   a4.CMORPH_V1.0_CRT.get_0.25deg-3HLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-HLY_<yyyymmddhh> : 0.25deg-HLY bias-corrected
c                                              CMORPH data hourly files
c OUTPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-3HLY_<yyyymmdd> : 0.25deg-3HLY bias-corrected
c                                             CMORPH data daily file
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   -----------:--------------------------------------
c   yyyymmdd   : target date
c   f_hly      : 0.25deg-HLY bias-corrected CMORPH data file name
c   f_3hr      : 0.25deg-3HLY bias-corrected CMORPH data file name
c   yyyy,mm,dd : target date
c   hh         : loop counter for hours
c   h3         : counter for 3-hour periods
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   ct()       : count of valid data points
c   rain()     : 0.25deg-HLY bias-corrected CMORPH data
c   ss()       : 0.25deg-3HLY bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_def_0.25deg_3HLY.lnx.f'
c
c   step 4 of 6 to generate CMORPH_V1.0_CRT dataset
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program a4_CMORPH_V10_CRT_get_ADJ_025deg_3HLY
      implicit none

      integer NX, NY
      parameter( NX=1440, NY=480 )

      character yyyymmdd*8, f_hly*46, f_3hr*45
      integer yyyy, mm, dd, hh, h3, ii, jj, ct(NX,NY), tt
      real rain(NX,NY), ss(NX,NY)

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|123456<--
      f_hly='../work/CMORPH_V1.0_ADJ_0.25deg-HLY_yyyymmddhh'
      f_3hr='../work/CMORPH_V1.0_ADJ_0.25deg-3HLY_yyyymmdd'

      if( iargc().ge.2 ) then
        f_hly(4:7)='redo'
        f_3hr(4:7)='redo'
      endif

      write(f_3hr(38:45), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c open 0.25deg-3HLY bias-corrected CMORPH data file for output
c-----------------------------------------------------------------------
      open(20, file=f_3hr, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c initialize arrays
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          ct(ii,jj)=0
          ss(ii,jj)=0.0
        enddo
      enddo

c-----------------------------------------------------------------------
c read in and sum up 0.25deg-HLY bias-corrected CMORPH data
c-----------------------------------------------------------------------
      do hh=1, 24

        write(f_hly(37:46), '(i4.4,3i2.2)') yyyy, mm, dd, hh-1

        open(10, file=f_hly, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_hly
        read(10, rec=1) rain
        close(10)

        do jj=1, NY
          do ii=1, NX
            if( rain(ii,jj).ge.0.0 ) then
              ct(ii,jj)=ct(ii,jj)+1
              ss(ii,jj)=ss(ii,jj)+rain(ii,jj)
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c output 0.25deg-3HLY data and reset arrays every 3 hours
c-----------------------------------------------------------------------
        if( mod(hh,3).eq.0 ) then

          do jj=1, NY
            do ii=1, NX
              if( ct(ii,jj).eq.0 ) then
                ss(ii,jj)=-999.0
              endif
            enddo
          enddo

c-----------------------------------------------------------------------
c truncate to 2 digits after decimal point
c-----------------------------------------------------------------------
          do jj=1, NY
            do ii=1, NX
              if( ss(ii,jj).le.-999.0 ) then
                ss(ii,jj)=-999.0
              else
                tt=nint(ss(ii,jj)*100.0)
                ss(ii,jj)=0.01*tt
              endif
            enddo
          enddo
          h3=hh/3
          write(20, rec=h3) ss

          do jj=1, NY
            do ii=1, NX
              ct(ii,jj)=0
              ss(ii,jj)=0.0
            enddo
          enddo

        endif

      enddo

      close(20)
      write(*, *) '  ==> ', f_3hr

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
