c@***h* CMORPH-CDR/a6.CMORPH_V1.0_CRT.get_ADJ_0.25deg-DLY_EOD.f
c
c=======================================================================
c PROGRAM
c   a6.CMORPH_V1.0_CRT.get_ADJ_0.25deg-DLY_EOD.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-DLY_EOD (daily with ending time matches daily
c   gauge data) bias-corrected CMORPH data from 0.25deg-HLY bias-corrected
c   CMORPH data
c
c-----------------------------------------------------------------------
c USAGE
c   a6.CMORPH_V1.0_CRT.get_ADJ_0.25deg-DLY_EOD.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_EOD_0.25deg_checked.lnx      : 0.25deg EOD mask
c   CMORPH_V1.0_EOD_0.25deg_checked.txt      : 0.25deg EOD stat data
c   CMORPH_V1.0_ADJ_0.25deg-HLY_<yyyymmddhh> : 0.25deg-HLY bias-corrected
c                                              CMORPH data hourly files
c
c OUTPUT FILES
c   CMORPH_V1.0_ADJ_0.25deg-DLY_EOD_<yyyymmdd> : 0.25deg-DLY_EOD bias-
c                                                corrected CMORPH data
c                                                daily file
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   NK         : number of lines in EOD stat data file
c   -----------:--------------------------------------
c   yyyymmdd   : target date
c   f_eod1     : 0.25deg EOD mask file name
c   f_eod2     : 0.25deg EOD stat data file name
c   f_hly      : 0.25deg-HLY bias-corrected CMORPH data file name
c   f_dly      : 0.25deg-DLY_EOD bias-corrected CMORPH data file name
c   yyyy,mm,dd : target date
c   days()     : number of days in each month
c   k          : loop counter for lines in EOD stat data file
c   endtime    : ending time
c   num        : number of grid boxes with EOD ending time as 'endtime'
c   ct()       : count of valid data points
c   hh         : loop counter for hours
c   hhs,hhe    : boundaries of a 24-hour period
c   y,m,d,h    : working year, month, day, hour
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   eod()      : 0.25deg EOD mask
c   rain()     : 0.25deg-HLY bias-corrected CMORPH data
c   ss()       : sum of 0.25deg-HLY CMORPH data over a 24-hour period
c   dly()      : 0.25deg-DLY_EOD bias-corrected CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_def_0.25deg_DLY_EOD.lnx.f'
c
c   step 6 of 6 to generate CMORPH_V1.0_CRT dataset
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program a6_CMORPH_V10_CRT_get_ADJ_025deg_DLY_EOD
      implicit none

      integer NX, NY, NK
      parameter( NX=1440, NY=480, NK=25 )

      character yyyymmdd*8, f_eod1*44, f_eod2*44, f_hly*46, f_dly*48
      integer yyyy, mm, dd, days(12), k, endtime, num
      integer ct(NX,NY), hh, hhs, hhe, y, m, d, h, ii, jj, tt
      real eod(NX,NY), rain(NX,NY), ss(NX,NY), dly(NX,NY)

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c adjust for leap-year
c-----------------------------------------------------------------------
      if( mod(yyyy,4).eq.0 )   days(2)=29
      if( mod(yyyy,100).eq.0 ) days(2)=28
      if( mod(yyyy,400).eq.0 ) days(2)=29

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c------------>123456789|123456789|123456789|123456789|1234<--
      f_eod1='../table/CMORPH_V1.0_EOD_0.25deg_checked.lnx'
      f_eod2='../table/CMORPH_V1.0_EOD_0.25deg_checked.txt'

c----------->123456789|123456789|123456789|123456789|12345678<--
      f_hly='../work/CMORPH_V1.0_ADJ_0.25deg-HLY_yyyymmddhh'
      f_dly='../work/CMORPH_V1.0_ADJ_0.25deg-DLY_EOD_yyyymmdd'

      if( iargc().ge.2 ) then
        f_hly(4:7)='redo'
        f_dly(4:7)='redo'
      endif

      write(f_dly(41:48), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c read in 0.25deg EOD mask
c-----------------------------------------------------------------------
      open(10, file=f_eod1, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_eod1
      read(10, rec=1) eod
      close(10)
      
c-----------------------------------------------------------------------
c open 0.25deg EOD stat data file for input
c-----------------------------------------------------------------------
      open(11, file=f_eod2, status='old',
     1    access='sequential', form='formatted')
      write(*, *) ' <--  ', f_eod2

c-----------------------------------------------------------------------
c initialize 0.25deg-DLY_EOD data array
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          dly(ii,jj)=-999.0
        enddo
      enddo

c-----------------------------------------------------------------------
c repeat for each line in EOD stat data file
c NOTE
c   there are 25 lines in the EOD stat data file
c   each line is in format: 'EOD  ee   nnnnn'
c   ee=ending time, nnnnn=number of grid boxes with EOD ending time='ee'
c-----------------------------------------------------------------------
      do k=1, NK

        read(11, '(5x,i2,i8)') endtime, num

        if( num.gt.0 ) then

          write(*, *) 'endtime: ', endtime

c-----------------------------------------------------------------------
c reset arrays
c-----------------------------------------------------------------------
          do jj=1, NY
            do ii=1, NX
              ct(ii,jj)=0
              ss(ii,jj)=0.0
            enddo
          enddo

c-----------------------------------------------------------------------
c generate 24-hour sum ending at 'endtime'
c-----------------------------------------------------------------------
          hhs=endtime-24
          hhe=endtime-1

          do hh=hhs, hhe

            h=hh
            d=dd
            m=mm
            y=yyyy
            if( hh.lt.0 ) then
              h=hh+24
              d=dd-1
              if( d.lt.1 ) then
                m=mm-1
                if( m.lt.1 ) then
                  m=12
                  y=yyyy-1
                endif
                d=days(m)
              endif
            else if( hh.ge.24 ) then
              h=hh-24
              d=dd+1
              if( d.gt.days(m) ) then
                d=1
                m=mm+1
                if( m.gt.12 ) then
                  m=1
                  y=yyyy+1
                endif
              endif
            endif

            write(f_hly(37:46), '(i4.4,3i2.2)') y, m, d, h

            open(12, file=f_hly, status='old',
     1          access='direct', recl=NX*NY*4)
            write(*, *) ' <--  ', f_hly
            read(12, rec=1) rain
            close(12)

            do jj=1, NY
              do ii=1, NX
                if( rain(ii,jj).ge.0.0 ) then
                  ct(ii,jj)=ct(ii,jj)+1
                  ss(ii,jj)=ss(ii,jj)+rain(ii,jj)
                endif
              enddo
            enddo

          enddo

c-----------------------------------------------------------------------
c select 24-hour sum as 0.25deg-DLY_EOD value according to EOD mask
c-----------------------------------------------------------------------
          do jj=1, NY
            do ii=1, NX

              if( ct(ii,jj).gt.0 ) then
                ss(ii,jj)=24.0*ss(ii,jj)/ct(ii,jj)
              else
                ss(ii,jj)=-999.0
              endif

              if( eod(ii,jj).eq.endtime ) then
                dly(ii,jj)=ss(ii,jj)
              endif

c-----------------------------------------------------------------------
c use endtime=24 sum over ocean
c-----------------------------------------------------------------------
              if( endtime.eq.24.and.eod(ii,jj).lt.0 ) then
                dly(ii,jj)=ss(ii,jj)
              endif

            enddo
          enddo

        endif

      enddo

c-----------------------------------------------------------------------
c truncate to 2 digits after decimal point
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          if( dly(ii,jj).le.-999.0 ) then
            dly(ii,jj)=-999.0
          else
            tt=nint(dly(ii,jj)*100.0)
            dly(ii,jj)=0.01*tt
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c output 0.25deg-DLY_EOD bias-corrected CMORPH data
c-----------------------------------------------------------------------
      open(20, file=f_dly, access='direct', recl=NX*NY*4)
      write(20, rec=1) dly
      close(20)
      write(*, *) '  ==> ', f_dly

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
