c@***h* CMORPH-CDR/g1.CMORPH_V1.0_CRT.get_FCT_GLB_0.25deg-HLY.f
c
c=======================================================================
c PROGRAM
c   g1.CMORPH_V1.0_CRT.get_FCT_GLB_0.25deg-HLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-HLY adjust factors over globe using 0.25deg-DLY
c   adjust factors over land and ocean for a 3-day processing window
c
c-----------------------------------------------------------------------
c USAGE
c   g1.CMOPRH_V1.0_CRT.get_FCT_GLB_0.25deg-HLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx   : 0.25deg land mask
c   CMORPH_V1.0_EOD_0.25deg_checked.lnx : 0.25deg EOD mask
c   FCT_LND_0.25deg-DLY_<yyyymmdd>      : 0.25deg-DLY adjust factors
c                                         over land
c   FCT_OCN_0.25deg-DLY_<yyyymmdd>      : 0.25deg-DLY adjust factors
c                                         over ocean
c
c OUTPUT FILES
c   FCT_GLB_0.25deg-HLY_<yyyymmdd> : 0.25deg-HLY adjust factors over globe
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX             : X-dimension (0-360) in 0.25deg grid
c   NY             : Y-dimension (60S-60N) in 0.25deg grid
c   N3Y            : number of days in 3 years
c   HW_D           : number of days in half of processing window
c   WIN_D          : number of days in processing window
c   ---------------:------------------------------------
c   yyyymmdd       : target date
c   f_msk          : 0.25deg land mask file name
c   f_eod          : 0.25deg EOD mask file name
c   f_lnd          : file name of 0.25deg-DLY adjust factors over land
c   f_ocn          : file name of 0.25deg-DLY adjust factors over ocean
c   f_glb          : file name of 0.25deg-HLY adjust factors over globe
c   yyyy,mm,dd     : target date
c   days()         : number of days in each month
c   y,m,d          : loop counters for years, months, days
c   td             : index in yyyy-mm-dd tables for target date
c   k              : counter for days in yyyy-mm-dd tables
c   k1,k2          : boundaries in yyyy-mm-dd tables for processing window
c   y0(),m0(),d0() : 3-year tables of yyyy-mm-dd
c   y1(),m1(),d1() : tables of yyyy-mm-dd for processing window
c   day_eod()      : day flag for EOD
c   dis            : searching distance
c   dis2           : distance^2 between working and target grid boxes
c   dis2max        : maximum dis2
c   ct             : count of valid data points
c   ii,i           : loop counters in X-direction
c   i1,i2          : X-boundaries for searching window
c   is             : X-index of working grid box
c   jj,j           : loop counters in Y-direction
c   j1,j2          : Y-boundaries for searching window
c   js             : Y-index of working grid box
c   hh             : loop counter for hours
c   he             : hour from EOD mask
c   mask()         : 0.25deg land mask
c   mask_smt()     : smoothed 0.25deg land mask (land/ocean boundary zone)
c   eod()          : 0.25deg EOD mask
c   ss             : sum of adjust factors
c   fct_lnd()      : 0.25deg-DLY adjust factors over land
c   fct_ocn()      : 0.25deg-DLY adjust factors over ocean
c   fct_d()        : 0.25deg-DLY adjust factors over globe over 3-day period
c   fct()          : working array for 0.25deg-HLY adjust factors
c   fct_glb()      : 0.25deg-HLY adjust factors over globe
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_0.25deg_hly.lnx.f'
c
c   step 1 of 2 to generate 0.25deg-HLY adjust factors over globe
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program g1_CMORPH_V10_CRT_get_FCT_GLB_025deg_HLY
      implicit none

      integer NX, NY, N3Y, HW_D, WIN_D
      parameter( NX=1440, NY=480, N3Y=1096, HW_D=1, WIN_D=1+2*HW_D )

      character yyyymmdd*8, f_msk*42, f_eod*44
      character f_lnd*36, f_ocn*36, f_glb*36
      integer yyyy, mm, dd, days(12), y, m, d, td, k, k1, k2
      integer y0(N3Y), m0(N3Y), d0(N3Y), y1(WIN_D), m1(WIN_D), d1(WIN_D)
      integer day_eod(24,NX,NY), dis, dis2, dis2max, ct
      integer ii, jj, i, i1, i2, is, j, j1, j2, js, hh, he
      real mask(NX,NY), eod(NX,NY), mask_smt(NX,NY), ss
      real fct_lnd(NX,NY), fct_ocn(NX,NY), fct_d(NX,NY,WIN_D)
      real fct(NX,NY), fct_glb(NX,NY)

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c create 3-year tables of yyyy-mm-dd
c-----------------------------------------------------------------------
      do k=1, N3Y
        y0(k)=-999
        m0(k)=-999
        d0(k)=-999
      enddo
      k=0
      do y=yyyy-1, yyyy+1
        days(2)=28
        if( mod(y,4).eq.0 )   days(2)=29
        if( mod(y,100).eq.0 ) days(2)=28
        if( mod(y,400).eq.0 ) days(2)=29
        do m=1, 12
          do d=1, days(m)
            k=k+1
            y0(k)=y
            m0(k)=m
            d0(k)=d
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c get index for target date
c-----------------------------------------------------------------------
      td=-999
      do k=1, N3Y
        if( y0(k).eq.yyyy.and.m0(k).eq.mm.and.d0(k).eq.dd ) then
          td=k
          exit
        endif
      enddo

c-----------------------------------------------------------------------
c get yyyy-mm-dd tables for processing window
c-----------------------------------------------------------------------
      if( yyyy.eq.1998.and.mm.eq.1.and.dd.le.HW_D ) then
        k1=365+1
        k2=365+WIN_D
      else
        k1=td-HW_D
        k2=td+HW_D
      endif
      d=0
      do k=k1, k2
        d=d+1
        y1(d)=y0(k)
        m1(d)=m0(k)
        d1(d)=d0(k)
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|1234<--
      f_msk='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_eod='../table/CMORPH_V1.0_EOD_0.25deg_checked.lnx'
      f_lnd='../work/FCT_LND_0.25deg-DLY_yyyymmdd'
      f_ocn='../work/FCT_OCN_0.25deg-DLY_yyyymmdd'
      f_glb='../work/FCT_GLB_0.25deg-HLY_yyyymmdd'

      if( iargc().ge.2 ) then
        f_lnd(4:7)='redo'
        f_ocn(4:7)='redo'
        f_glb(4:7)='redo'
      endif

      write(f_glb(29:36), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c read in 0.25deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_msk
      read(10, rec=1) mask
      close(10)

c-----------------------------------------------------------------------
c mark land/ocean boundary zone
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          if( mask(ii,jj).ne.0.0 ) then
            mask_smt(ii,jj)=1.0
          else
            dis=2
            dis2max=dis*dis
            j1=jj-dis
            if( j1.lt.1 ) j1=1
            j2=jj+dis
            if( j2.gt.NY ) j2=NY
            i1=ii-dis
            i2=ii+dis
            ct=0
            do j=j1, j2
              do i=i1, i2
                dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                if( dis2.le.dis2max ) then
                  js=j
                  if( i.lt.1 ) then
                    is=i+NX
                  elseif( i.ge.1.and.i.le.NX ) then
                    is=i
                  else
                    is=i-NX
                  endif
                  if( mask(is,js).ne.0.0 ) ct=ct+1
                endif
              enddo
            enddo
            if( ct.gt.0 ) then
              mask_smt(ii,jj)=0.5
            else
              mask_smt(ii,jj)=0.0
            endif
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c read in 0.25deg EOD mask
c-----------------------------------------------------------------------
      open(11, file=f_eod, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_eod
      read(11, rec=1) eod
      close(11)

c-----------------------------------------------------------------------
c set day flag for EOD
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX

          do hh=1, 24
            day_eod(hh,ii,jj)=-999
          enddo

          if( eod(ii,jj).lt.0.0 ) then
            do hh=1, 24
              day_eod(hh,ii,jj)=0
            enddo
          else
            he=nint(eod(ii,jj))
            if( he.lt.1 ) then
              write(*, *) '#### wrong EOD: ', ii, jj, he
              stop
            elseif( he.ge.1.and.he.le.24 ) then
              do hh=1, he
                day_eod(hh,ii,jj)=0
              enddo
              if( he.lt.24 ) then
                do hh=he, 24
                  day_eod(hh,ii,jj)=1
                enddo
              endif
            elseif( he.le.36 ) then
              do hh=1, he-24
                day_eod(hh,ii,jj)=-1
              enddo
              do hh=he-23, 24
                day_eod(hh,ii,jj)=0
              enddo
            else
              write(*, *) '#### wrong EOD: ', ii, jj, he
              stop
            endif
          endif

        enddo
      enddo

c-----------------------------------------------------------------------
c initialize arrays
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          fct(ii,jj)=-999.0
          do d=1, WIN_D
            fct_d(ii,jj,d)=-999.0
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c output missing values
c-----------------------------------------------------------------------
c      do hh=1, 24
c        write(20, rec=hh) fct
c      enddo

c-----------------------------------------------------------------------
c read in 0.25deg-DLY adjust factors over land and ocean
c-----------------------------------------------------------------------
      do d=1, WIN_D

        write(f_lnd(29:36), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

        write(f_ocn(29:36), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

        open(12, file=f_lnd, status='old',
     1      access='direct', recl=NX*NY*4)
        open(13, file=f_ocn, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_lnd
        write(*, *) ' <--  ', f_ocn
        read(12, rec=1) fct_lnd
        read(13, rec=1) fct_ocn
        close(12)
        close(13)

        do jj=1, NY
          do ii=1, NX
            if( mask(ii,jj).ne.0.0 ) then
              fct_d(ii,jj,d)=fct_lnd(ii,jj)
            else
              fct_d(ii,jj,d)=fct_ocn(ii,jj)
            endif
          enddo
        enddo

      enddo

c-----------------------------------------------------------------------
c check for missing values
c-----------------------------------------------------------------------
      do d=1, WIN_D
        do jj=1, NY
          do ii=1, NX
            if( fct_d(ii,jj,d).lt.0.0 ) then
              fct_d(ii,jj,d)=1.0
            endif
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c open file of 0.25deg-HLY adjust factors over globe for ouput
c-----------------------------------------------------------------------
      open(20, file=f_glb, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c repeat for each hour in target date
c-----------------------------------------------------------------------
      do hh=1, 24

c-----------------------------------------------------------------------
c assign 0.25deg-DLY values as 0.25deg-HLY values
c-----------------------------------------------------------------------
        do jj=1, NY
          do ii=1, NX
            d=day_eod(hh,ii,jj)+2
            if( d.lt.1.or.d.gt.WIN_D ) then
              write(*, *) '#### wrong day: ', ii, jj, d
              stop
            endif
            fct(ii,jj)=fct_d(ii,jj,d)
            if( fct(ii,jj).lt.0.2 ) then
              fct(ii,jj)=0.2
            endif
            if( fct(ii,jj).gt.5.0 ) then
              fct(ii,jj)=5.0
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c smooth land/ocean boundary zone
c-----------------------------------------------------------------------
        do jj=1, NY
          do ii=1, NX
            if( mask_smt(ii,jj).ne.0.5 ) then
              fct_glb(ii,jj)=fct(ii,jj)
            else
              dis=3
 100          dis2max=dis*dis
              j1=jj-dis
              if( j1.lt.1 ) j1=1
              j2=jj+dis
              if( j2.gt.NY ) j2=NY
              i1=ii-dis
              i2=ii+dis
              ct=0
              ss=0.0
              do j=j1, j2
                do i=i1, i2
                  dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                  if( dis2.le.dis2max ) then
                    js=j
                    if( i.lt.1 ) then
                      is=i+NX
                    elseif( i.ge.1.and.i.le.NX ) then
                      is=i
                    else
                      is=i-NX
                    endif
                    if( fct(is,js).ge.0.0 ) then
                      ct=ct+1
                      ss=ss+fct(is,js)
                    endif
                  endif
                enddo
              enddo
              if( ct.ge.5 ) then
                fct_glb(ii,jj)=ss/ct
              else
                dis=dis+1
                goto 100
              endif
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c output 0.25deg-HLY adjust factors over globe
c-----------------------------------------------------------------------
        write(20, rec=hh) fct_glb

      enddo

      close(20)
      write(*, *) '  ==> ', f_glb

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
