c***h* CMORPH-CDR/g2.CMORPH_V1.0_CRT.get_FCT_SMT_0.25deg-HLY.f
c
c=======================================================================
c PROGRAM
c   g2.CMORPH_V1.0_CRT.get_FCT_SMT_0.25deg-HLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to perform temporal smoothing on 0.25deg-HLY adjust factors over
c   globe using a 9-hour smoothing window
c
c-----------------------------------------------------------------------
c USAGE
c   g2.CMOPRH_V1.0_CRT.get_FCT_SMT_0.25deg-HLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   FCT_GLB_0.25deg-HLY_<yyyymmdd> : 0.25deg-HLY adjust factors
c
c OUTPUT FILES
c   FCT_SMT_0.25deg-HLY_<yyyymmdd> : smoothed 0.25deg-HLY adjust factors
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX             : X-dimension (0-360) in 0.25deg grid
c   NY             : Y-dimension (60S-60N) in 0.25deg grid
c   N3Y            : number of days in 3 years
c   NH             : number of hours in a day
c   HW_D           : number of days in half of processing window
c   WIN_D          : number of days in processing window
c   HW_H           : number of hours in half of smoothing window
c   ---------------:------------------------------------
c   yyyymmdd       : target date
c   f_glb          : file name of 0.25deg-HLY adjust factors
c   f_smt          : file name of smoothed 0.25deg-HLY adjust factors
c   yyyy,mm,dd     : target date
c   days()         : number of days in each month
c   y,m,d          : loop counters for years, months, days
c   td             : index in yyyy-mm-dd tables for target date
c   k              : counter for days in yyyy-mm-dd tables
c   k1,k2          : boundaries in yyyy-mm-dd tables for processing window
c   y0(),m0(),d0() : 3-year tables of yyyy-mm-dd
c   y1(),m1(),d1() : tables of yyyy-mm-dd for processing window
c   dis            : searching distance
c   dis2           : distance^2 between working and target grid boxes
c   dis2max        : maximum dis2
c   ii             : loop counter in X-direction
c   jj             : loop counter in Y-direction
c   hh,h           : loop counters for hours
c   h1,h2          : boundaries for hours
c   ct()           : count of valid data points
c   ss()           : sum of adjust factors
c   fct()          : 0.25deg-HLY adjust factors
c   fct_h()        : 0.25deg-HLY adjust factors over 32-hour period
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_0.25deg_hly_smooth.lnx.f'
c
c   step 2 of 2to generate 0.25deg-HLY adjust factors over globe
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program g2_CMORPH_V10_CRT_get_FCT_SMT_025deg_HLY
      implicit none

      integer NX, NY, N3Y, NH, HW_D, WIN_D, HW_H
      parameter( NX=1440, NY=480, N3Y=1096, NH=24 )
      parameter( HW_D=1, WIN_D=2*HW_D+1, HW_H=4 )

      character yyyymmdd*8, f_glb*36, f_smt*36
      integer yyyy, mm, dd, days(12), y, m, d, td, k, k1, k2
      integer y0(N3Y), m0(N3Y), d0(N3Y), y1(WIN_D), m1(WIN_D), d1(WIN_D)
      integer ii, jj, hh, h, h1, h2, ct(NX,NY)
      real ss(NX,NY), fct(NX,NY), fct_h(NX,NY,NH+2*HW_H)

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c create 3-year tables of yyyy-mm-dd
c-----------------------------------------------------------------------
      do k=1, N3Y
        y0(k)=-999
        m0(k)=-999
        d0(k)=-999
      enddo
      k=0
      do y=yyyy-1, yyyy+1
        days(2)=28
        if( mod(y,4).eq.0 )   days(2)=29
        if( mod(y,100).eq.0 ) days(2)=28
        if( mod(y,400).eq.0 ) days(2)=29
        do m=1, 12
          do d=1, days(m)
            k=k+1
            y0(k)=y
            m0(k)=m
            d0(k)=d
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c get index for target date
c-----------------------------------------------------------------------
      td=-999
      do k=1, N3Y
        if( y0(k).eq.yyyy.and.m0(k).eq.mm.and.d0(k).eq.dd ) then
          td=k
          exit
        endif
      enddo

c-----------------------------------------------------------------------
c get yyyy-mm-dd tables for 3-day processing window
c-----------------------------------------------------------------------
      if( yyyy.eq.1998.and.mm.eq.1.and.dd.le.HW_D ) then
        k1=365+1
        k2=365+WIN_D
      else
        k1=td-HW_D
        k2=td+HW_D
      endif
      d=0
      do k=k1, k2
        d=d+1
        y1(d)=y0(k)
        m1(d)=m0(k)
        d1(d)=d0(k)
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456<--
      f_glb='../work/FCT_GLB_0.25deg-HLY_yyyymmdd'
      f_smt='../work/FCT_SMT_0.25deg-HLY_yyyymmdd'

      if( iargc().ge.2 ) then
        f_glb(4:7)='redo'
        f_smt(4:7)='redo'
      endif

      write(f_smt(29:36), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c read in 32 hours (last 4 hours from previous day, all 24 hours from
c target date, first 4 hours from next day) 0.25deg-HLY adjust factors
c-----------------------------------------------------------------------
      hh=0
      do d=1, WIN_D

        write(f_glb(29:36), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)
        open(10, file=f_glb, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_glb

        if( d.eq.1 ) then
          h1=NH-HW_H+1
          h2=NH
        else if( d.eq.2 ) then
          h1=1
          h2=NH
        else if( d.eq.3 ) then
          h1=1
          h2=HW_H
        else
          write(*, *) '#### error in data record'
          stop
        endif

        do h=h1, h2
          read(10, rec=h) fct
          hh=hh+1
          do jj=1, NY
            do ii=1, NX
              fct_h(ii,jj,hh)=fct(ii,jj)
            enddo
          enddo
        enddo

        close(10)

      enddo

c-----------------------------------------------------------------------
c open file of 0.25deg-HLY smoothed adjust factors over globe for output
c-----------------------------------------------------------------------
      open(20, file=f_smt, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c perform temporal smoothing over a 9-hour smoothing window
c-----------------------------------------------------------------------
      do hh=1, NH

        h1=hh
        h2=hh+2*HW_H

        do jj=1, NY
          do ii=1, NX
            ct(ii,jj)=0
            ss(ii,jj)=0.0
          enddo
        enddo

        do h=h1, h2
          do jj=1, NY
            do ii=1, NX
              if( fct_h(ii,jj,h).gt.0.0 ) then
                ct(ii,jj)=ct(ii,jj)+1
                ss(ii,jj)=ss(ii,jj)+fct_h(ii,jj,h)
              endif
            enddo
          enddo
        enddo

        do jj=1, NY
          do ii=1, NX
            if( ct(ii,jj).gt.0 ) then
              fct(ii,jj)=ss(ii,jj)/ct(ii,jj)
            else
              fct(ii,jj)=1.0
            endif
          enddo
        enddo

        write(20, rec=hh) fct

      enddo

      close(20)
      write(*, *) '  ==> ', f_smt

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
