c@***h* CMORPH-CDR/l1.CMORPH_V1.0_CRT.get_HST_0.25deg-DLY_EOD.f
c
c=======================================================================
c PROGRAM
c   l1.CMORPH_V1.0_CRT.get_HST_0.25deg-DLY_EOD.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to remove bias in CMORPH estimates through historical bias
c   correction tables
c
c-----------------------------------------------------------------------
c USAGE
c   l1.CMORPH_V1.0_CRT.get_HST_0.25deg-DLY_EOD.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx          : 0.25deg land mask
c   CMORPH_BIAS_CRT_HS_V1.0_STAT.lnx.DAY<jjj>  : STAT table
c   CMORPH_BIAS_CRT_HS_V1.0_CPDF.lnx.DAY<jjj>  : CPDF table
c   CMORPH_V1.0_RAW_0.25deg-DLY_EOD_<yyyymmdd> : raw CMORPH data
c
c OUTPUT FILES
c   CMORPH_ADJ_HST_0.25deg-DLY_EOD_<yyyymmdd> : adjusted CMORPH data
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX          : X-dimension (0-360) in 0.25deg grid
c   NY          : Y-dimension (60S-60N) in 0.25deg grid
c   NC          : number of classes in CPDF tables
c   A, B        : set upper limit of adjust ratio as a function of
c                 precip intensity: rmax=a/(R+b)
c   ------------:-------------------------------
c   yyyymmdd    : target date
c   f_mask      : land mask file name
c   f_stat      : STAT correction table file name
c   f_cpdf      : CPDF correction table file name
c   f_raw       : raw CMORPH data file name
c   f_adj       : adjusted CMORPH data file name
c   yyyy        : year
c   mm          : month
c   dd          : day
c   days()      : number of days in each month
c   jjj         : Julian day (non-leap-year)
c   m           : loop counter for months
c   ii          : loop counter in X-direction 
c   jj          : loop counter in Y-direction
c   kk          : counter for land grid boxes
c   kk_land()   : counter for land grid boxes
c   mask()      : land mask in INTEGER format
c   cc          : loop counter for CPDF classes
c   cls         : class
c   rmask()     : land mask in REAl format
c   pdf_rain()  : CPDF data for precip
c   pdf_ratio() : CPDF data for ratio
c   fct()       : correctiopn factor
c   raw()       : arw CMORPH data
c   adj()       : adjusted CMORPH data
c   rat         : ratio for working point
c   rat0        : ratio for the lower class
c   rat1        : ratio for the higher class
c   rain        : rain for working point
c   rain0       : rain for the lower class
c   rain1       : rain for the higher class
c   rmax        : upper limit of ratio
c   ff          : correction factor
c   
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_HIST_ADJ.f'
c
c   step 1 of 4 to generate 0.25deg-DLY adjust factors over land
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program l1_CMORPH_V10_CRT_get_HST_025deg_DLY_EOD
      implicit none

      integer NX, NY, NC
      real A, B
      parameter( NX=1440, NY=480, NC=100, A=60.0, B=10.0 )

      character yyyymmdd*8, f_mask*42, f_stat*48, f_cpdf*48
      character f_raw*48, f_adj*47
      integer yyyy, mm, dd, days(12), jjj, m
      integer ii, jj, kk, kk_land(NX,NY), mask(NX,NY), cc, cls
      real rmask(NX,NY), pdf_rain(NC), pdf_ratio(NC), fct(NX,NY)
      real raw(NX,NY), adj(NX,NY)
      real rat, rat0, rat1, rain, rain0, rain1, rmax, ff

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c get Julian day (non-leap-year)
c-----------------------------------------------------------------------
      jjj=dd
      do m=1, mm-1
        jjj=jjj+days(m)
      enddo

c-----------------------------------------------------------------------
c adjust for Feb. 29
c-----------------------------------------------------------------------
      if( mm.eq.2.and.dd.eq.29 ) jjj=59

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c------------>123456789|123456789|123456789|123456789|12345678<---
      f_mask='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_stat='../table/CMORPH_BIAS_CRT_HS_V1.0_STAT.lnx.DAYj??'
      f_cpdf='../table/CMORPH_BIAS_CRT_HS_V1.0_CPDF.lnx.DAYj??'

c----------->123456789|123456789|123456789|123456789|12345678<--
      f_raw='../work/CMORPH_V1.0_RAW_0.25deg-DLY_EOD_y???m?d?'
      f_adj='../work/CMORPH_ADJ_HST_0.25deg-DLY_EOD_y???m?d?'

      if( iargc().ge.2 ) then
        f_raw(4:7)='redo'
        f_adj(4:7)='redo'
      endif

      write(f_stat(46:48), '(i3.3)') jjj
      write(f_cpdf(46:48), '(i3.3)') jjj

      write(f_raw(41:48), '(i4.4,2i2.2)') yyyy, mm, dd

      write(f_adj(40:47), '(i4.4,2i2.2)') yyyy, mm, dd

      open(10, file=f_mask, status='old',
     1    access='direct', recl=NX*NY*4)
      open(11, file=f_stat, status='old',
     1    access='direct', recl=NX*NY*4)
      open(12, file=f_cpdf, status='old',
     1    access='direct', recl=NC*4)
      open(13, file=f_raw, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_mask
      write(*, *) ' <--  ', f_stat
      write(*, *) ' <--  ', f_cpdf
      write(*, *) ' <--  ', f_raw

      open(20, file=f_adj, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c read in land mask
c-----------------------------------------------------------------------
      read(10, rec=1) rmask
      close(10)

      do jj=1, NY
        do ii=1, NX
          mask(ii,jj)=nint(rmask(ii,jj))
        enddo
      enddo

c-----------------------------------------------------------------------
c check for land grid boxes
c-----------------------------------------------------------------------
      kk=0
      do jj=1, NY
        do ii=1, NX
          kk_land(ii,jj)=-999
          if( mask(ii,jj).eq.1 ) then
            kk=kk+1
            kk_land(ii,jj)=kk
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c read in correction factor
c-----------------------------------------------------------------------
      read(11, rec=3) fct
      close(11)

c-----------------------------------------------------------------------
c read in raw CMORPH data
c-----------------------------------------------------------------------
      read(13, rec=1) raw
      close(13)

c-----------------------------------------------------------------------
c initialize adjust CMORPH array
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          adj(ii,jj)=raw(ii,jj)
        enddo
      enddo

c-----------------------------------------------------------------------
c correct bias through CPDF matching
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX

          if( kk_land(ii,jj).gt.0 ) then
            if( raw(ii,jj).gt.0.0 ) then

              kk=kk_land(ii,jj)
              read(12, rec=(kk-1)*2+1) pdf_rain
              read(12, rec=(kk-1)*2+2) pdf_ratio

              rat=-999.0
              if( raw(ii,jj).le.pdf_rain(1) ) then
                cls=1
                rat=pdf_ratio(1)
              else if( raw(ii,jj).gt.pdf_rain(NC) ) then
                cls=NC
                rat=pdf_ratio(NC)
              else
                cls=1
                do cc=1, NC-1
                  if( raw(ii,jj).ge.pdf_rain(cc).and.
     1                raw(ii,jj).lt.pdf_rain(cc+1) ) then
                    cls=cc
                    exit
                  endif
                enddo
                rat0=pdf_ratio(cls)
                rat1=pdf_ratio(cls+1)
                rain0=pdf_rain(cls)
                rain1=pdf_rain(cls+1)
                rain=raw(ii,jj)
                if( rat0.ge.0.0.and.rat1.ge.0.0.and.
     1              rain0.ge.0.0.and.rain1.ge.0.0 ) then
                  if( abs(rain1-rain0).lt.1.0e-6 ) then
                    rat=rat0
                  else
                    rat=rat0+(rat1-rat0)*(rain-rain0)/(rain1-rain0)
                  endif
                else
                  rat=1.0
                endif
              endif

              if( rat.lt.0.0 ) then
                rat=1.0
              else if( rat.gt.0.0 ) then
                rmax=A/(raw(ii,jj)+B)
                if( rmax.lt.1.5 ) then
                  rmax=1.5
                else if( rmax.gt.5.0 ) then
                  rmax=5.0
                endif
                if( rat.gt.rmax ) then
                  rat=rmax
                endif
              endif

              adj(ii,jj)=rat*raw(ii,jj)

              if( fct(ii,jj).gt.0.0 ) then
                if( fct(ii,jj).lt.0.95 ) then
                  ff=0.95
                else if( fct(ii,jj).le.1.05 ) then
                  ff=fct(ii,jj)
                else
                  ff=1.05
                endif
                adj(ii,jj)=ff*adj(ii,jj)
              endif

            endif
          endif

        enddo
      enddo

      close(12)

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
      write(20, rec=1) adj
      close(20)
      write(*, *) '  ==> ', f_adj

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
