c@***h* CMORPH-CDR/l2.CMORPH_V1.0_CRT.get_INPUTS_center_31days.f
c
c=======================================================================
c PROGRAM
c   l2.CMORPH_V1.0_CRT.get_INPUTS_center_31days.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to create a binary data file (as input for next step) containing
c   0.25deg gauge data and 0.25deg-DLY_EOD HST-adjusted CMORPH data
c   for 31 days centering at the target date
c
c-----------------------------------------------------------------------
c USAGE
c   l2.CMORPH_V1.0_CRT.get_INPUTS_center_31days.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   PRCP_CU_GAUGE_V1.0GLB_0.25deg.lnx.<yyyymmdd>.RT : 0.25deg gauge data
c   CMORPH_ADJ_HST_0.25deg-DLY_EOD_<yyyymmdd>       : 0.25deg-DLY_EOD
c                                                     HST-adjusted
c                                                     CMORPH data
c
c OUTPUT FILES
c   INPUTS_center_31days.<yyyymmdd> : combined 31-day data
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX             : X-dimension (0-360) in 0.25deg grid
c   NY             : Y-dimension (60S-60N) in 0.25deg grid
c   NY1            : Y-dimension (90S-90N) in 0.25deg grid
c   N3Y            : number of day sin 3 years
c   WIN            : half of processing window
c   ND             : number of days in processing window
c   ---------------:------------------------------------
c   yyyymmdd       : target date
c   f_work         : file name of combined 31-day data
c   f_gag          : 0.25deg gauge data file name
c   f_hst          : 0.25deg-DLY_EOD HST-adjusted CMORPH data file name
c   yyyy,mm,dd     : target date
c   days()         : number of days in each month
c   y,m,d          : loop counters for years, months, days
c   td             : index in yyyy-mm-dd tables for target date
c   k              : counter for days in yyyy-mm-dd tables
c   k1,k2          : boundaries in yyyy-mm-dd tables for processing window
c   ii             : loop counter in X-direction
c   jj             : loop counter in Y-direction
c   y0(),m0(),d0() : 3-year tables of yyyy-mm-dd
c   y1(),m1(),d1() : yyyy-mm-dd tables for processing window
c   gage1()        : 90S-90N 0.25deg gauge precip data
c   samp1()        : 90S-90N 0.25deg gauge sampling data
c   gage()         : 60S-60N 0.25deg gauge precip data
c   samp()         : 60S-60N 0.25deg gauge sampling data
c   sate()         : 0.25deg-DLY_EOD HST-adjusted CMORPH data
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_get_inp.lnx.f'
c
c   step 2 of 4 to generate 0.25deg-DLY adjust factors over land
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program l2_CMORPH_V10_CRT_get_INPUTS_center_31days
      implicit none

      integer NX, NY, NY1, N3Y, WIN, ND
      parameter( NX=1440, NY=480, NY1=720 )
      parameter( N3Y=1096, WIN=15, ND=2*WIN+1 )

      character yyyymmdd*8, f_work*37, f_gag0*50, f_gagr*53, f_hst*47
      integer yyyy, mm, dd, days(12), y, m, d, td, k, k1, k2, ii, jj
      integer y0(N3Y), m0(N3Y), d0(N3Y), y1(ND), m1(ND), d1(ND)
      real gage1(NX,NY1), samp1(NX,NY1), gage(NX,NY), samp(NX,NY)
      real sate(NX,NY)
      logical gag0_there, gagr_there

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c create 3-year tables of yyyy-mm-dd
c-----------------------------------------------------------------------
      do k=1, N3Y
        y0(k)=-999
        m0(k)=-999
        d0(k)=-999
      enddo
      k=0
      do y=yyyy-1, yyyy+1
        days(2)=28
        if( mod(y,4).eq.0 )   days(2)=29
        if( mod(y,100).eq.0 ) days(2)=28
        if( mod(y,400).eq.0 ) days(2)=29
        do m=1, 12
          do d=1, days(m)
            k=k+1
            y0(k)=y
            m0(k)=m
            d0(k)=d
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c find index for target date
c-----------------------------------------------------------------------
      td=-999
      do k=1, N3Y
        if( y0(k).eq.yyyy.and.m0(k).eq.mm.and.d0(k).eq.dd ) then
          td=k
          exit
        endif
      enddo

c-----------------------------------------------------------------------
c get yyyy-mm-dd tables for processing window
c-----------------------------------------------------------------------
      if( yyyy.eq.1998.and.mm.eq.1.and.dd.le.WIN ) then
        k1=365+1
        k2=365+ND
      else
        k1=td-WIN
        k2=td+WIN
      endif
      d=0
      do k=k1, k2
        d=d+1
        y1(d)=y0(k)
        m1(d)=m0(k)
        d1(d)=d0(k)
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c------------>123456789|123456789|123456789|123456789|123456789|123<--
      f_work='../work/INPUTS_center_31days.yyyymmdd'
      f_gag0='../work/PRCP_CU_GAUGE_V1.0GLB_0.25deg.lnx.yyyymmdd'
      f_gagr='../work/PRCP_CU_GAUGE_V1.0GLB_0.25deg.lnx.yyyymmdd.RT'
       f_hst='../work/CMORPH_ADJ_HST_0.25deg-DLY_EOD_yyyymmdd'

      if( iargc().ge.2 ) then
        f_work(4:7)='redo'
        f_gag0(4:7)='redo'
        f_gagr(4:7)='redo'
         f_hst(4:7)='redo'
      endif

      write(f_work(30:37), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c open output file
c-----------------------------------------------------------------------
      open(20, file=f_work, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c repeat for each day in processing window
c-----------------------------------------------------------------------
      do d=1, ND

        write(f_gag0(43:50), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)
        write(f_gagr(43:50), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

        write(f_hst(40:47), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

c-----------------------------------------------------------------------
c read in 90S-90N gauage data
c-----------------------------------------------------------------------
        inquire(file=f_gag0, exist=gag0_there)
        if( gag0_there ) then
          open(11, file=f_gag0, status='old',
     1        access='direct', recl=NX*NY1*4)
          write(*, *) ' <--  ', f_gag0
        else
          open(11, file=f_gagr, status='old',
     1        access='direct', recl=NX*NY1*4)
          write(*, *) ' <--  ', f_gagr
        endif
        read(11, rec=1) gage1
        read(11, rec=2) samp1
        close(11)

c-----------------------------------------------------------------------
c read in HST-adjusted CMORPH data
c-----------------------------------------------------------------------
        open(12, file=f_hst, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_hst
        read(12, rec=1) sate
        close(12)

c-----------------------------------------------------------------------
c extract 60S-60N gauge data and adjust CMORPH data
c-----------------------------------------------------------------------
        do jj=1, NY
          do ii=1, NX
            gage(ii,jj)=gage1(ii,jj+120)
            if( gage(ii,jj).ge.0.0 ) then
              gage(ii,jj)=0.1*gage(ii,jj)
              if( gage(ii,jj).lt.0.05 ) then
                gage(ii,jj)=0.0
              endif
            else
              gage(ii,jj)=-999.0
            endif
            samp(ii,jj)=samp1(ii,jj+120)

            if( sate(ii,jj).gt.0.0.and.sate(ii,jj).lt.0.05 ) then
              sate(ii,jj)=0.0
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
        write(20, rec=(d-1)*3+1) gage
        write(20, rec=(d-1)*3+2) samp
        write(20, rec=(d-1)*3+3) sate

      enddo

      close(20)
      write(*, *) '  ==> ', f_work

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
