c@***h* CMORPH-CDR/l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.f
c
c=======================================================================
c PROGRAM
c   l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to remove bias in 0.25deg-DLY_EOD HST-adjusted CMORPH data over land
c   through PDF matching against concurrent and colocated 0.25deg gauge
c   data for 31 days centering at target date
c
c-----------------------------------------------------------------------
c USAGE
c   l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx : 0.25deg land mask
c   INPUTS_center_31days.<yyyymmdd>   : combined 31-day data
c
c OUTPUT FILES
c   CMORPH_ADJ_LND_0.25deg-DLY_EOD_<yyyymmdd> : 0.25deg-DLY_EOD land-
c                                               adjusted CMORPH data
c
c INCLUDED FILES
c   l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.h : parameters, variables
c                                                  and common data blocks
c
c-----------------------------------------------------------------------
c SUBROUTINES USED
c   PDF_BIAS_V0805_main() : to correct bias through PDF matching
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optioanl flag for re-processing
c
c LOCAL VARIABLES
c   nx025      : X-dimension (0-360) in 0.25deg grid (defined in .h file)
c   ny025      : Y-dimension (60S-60N) in 0.25deg grid (defined in .h file)
c   -----------:-----------------------------------------------------------
c   yyyymmdd   : target date
c   f_mask     : 0.25deg land mask file name
c   f_work     : combined 31-day data file name
c   f_land     : 0.25deg-DLY_EOD land-adjusted CMORPH data file name
c   yyyy,mm,dd : target date
c   rmask025() : 0.25deg land mask in REAL format
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   mask025()  : 0.25deg land mask in INTEGER format (defined in .h file)
c   kinp1      : gauge precip data record number for target date
c   kinp2      : gauge sampling data record number for target date
c   kinp3      : HST-adjusted CMORPH data record number for target date
c   rgage()    : 0.25deg gauge precip data (defined in .h file)
c   sgage()    : 0.25deg gauge sampling data (defined in .h file)
c   rsate()    : 0.25deg-DLY_EOD HST-adjusted CMORPH data (defined in .h file)
c   kstatus    : return status of subroutine
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'cmorph_adj_land_31day.f'
c
c   step 3 of 4 to generate 0.25deg-DLY adjust factors over land
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program l3_CMORPH_V10_CRT_get_LND_025deg_DLY_EOD

      include 'l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.h'

      character yyyymmdd*8, f_mask*42, f_work*37, f_land*47
      integer yyyy, mm, dd
      dimension rmask025(nx025,ny025)

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c------------>123456789|123456789|123456789|123456789|1234567<--
      f_mask='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_work='../work/INPUTS_center_31days.yyyymmdd'
      f_land='../work/CMORPH_ADJ_LND_0.25deg-DLY_EOD_yyyymmdd'

      if( iargc().ge.2 ) then
        f_work(4:7)='redo'
        f_land(4:7)='redo'
      endif

      write(f_work(30:37), '(i4.4,2i2.2)') yyyy, mm, dd
      write(f_land(40:47), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c raed in 0.25deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_mask, status='old',
     1    access='direct', recl=nx025*ny025*4)
      write(*, *) ' <--  ', f_mask
      read(10, rec=1) rmask025
      close(10)
      do jj=1, ny025
        do ii=1, nx025
          mask025(ii,jj)=nint(rmask025(ii,jj))
        enddo
      enddo

c-----------------------------------------------------------------------
c open combined 31-day data file
c-----------------------------------------------------------------------
      open(40, file=f_work, status='old',
     1    access='direct', recl=nx025*ny025*4)
      write(*, *) ' <--  ', f_work

c-----------------------------------------------------------------------
c read in gauge data and HST-adjusted CMORPH data for target date
c NOTE:
c   seemed rgage() and sgage() were not used
c-----------------------------------------------------------------------
      if( yyyy.eq.1998.and.mm.eq.1.and.dd.le.16 ) then
        kday=dd-1               ! CMORPH DLY EOD start at 01/02/1998
        kinp1=(kday-1)*3+1
        kinp2=(kday-1)*3+2
        kinp3=(kday-1)*3+3
      else
        kinp1=15*3+1
        kinp2=15*3+2
        kinp3=15*3+3
      endif
      read(40, rec=kinp1) rgage
      read(40, rec=kinp2) sgage
      read(40, rec=kinp3) rsate

c-----------------------------------------------------------------------
c open output file
c-----------------------------------------------------------------------
      open(50, file=f_land, access='direct', recl=nx025*ny025*4)

c-----------------------------------------------------------------------
c initialize arrays
c-----------------------------------------------------------------------
      do jj=1, ny025
        do ii=1, nx025
          ratio(ii,jj)=-999.0
          fct(ii,jj)=-999.0
        enddo
      enddo

c-----------------------------------------------------------------------
c correct bias through PDF matching
c-----------------------------------------------------------------------
      kstatus=0
      call PDF_BIAS_V0805_main(kstatus)

c-----------------------------------------------------------------------
c check return status
c-----------------------------------------------------------------------
      if( kstatus.eq.0 ) then
        write(*, *) '## finished correcting CMORPH bias'
      elseif( kstatus.eq.5 ) then
        write(*, *) '## no bias correction performed'
      else
        write(*, *) '## CMORPH missing'
      endif
 
      close(40)
      close(50)
      write(*, *) '  ==> ', f_land

      stop 
      end  
c
c=====|==|=========|=========|=========|=========|=========|=========|==
c
c     #######################################################################
c     ###                                                                 ###
c     ###  Subroutine     :                                               ###
c     ###                                                                 ###
c     ###       name      :                                               ###
c     ###                     PDF_BIAS_V0805_main ( kstatus )             ###
c     ###                                                                 ###
c     ###      function   :                                               ###
c     ###                     to correct the CMORPH bias through          ###
c     ###                     PDF matching                                ###
c     ###                                                                 ###
c     ###      inputs     :                                               ###
c     ###                     unit 30   =  31-day daily fields            ###
c     ###                                                                 ###
c     ###                                                                 ###
c     ###      outputs    :                                               ###
c     ###                     kstatus   =  correction status              ###
c     ###                                  0  == corrected                ###
c     ###                                  9  == not corrected            ###
c     ###                     unit 50   =  corrected CMORPH (rec=5)       ###
c     ###                                                                 ###
c     ###                                                                 ###
c     #######################################################################
c
      subroutine  PDF_BIAS_V0805_main  ( kstatus )
c
      include 'l3.CMORPH_V1.0_CRT.get_LND_0.25deg-DLY_EOD.h'

      dimension
c            1.  gauge/satellite data
     #           rain (nx025,ny025), samp (nx025,ny025),
     #           sate (nx025,ny025),
c            2.  Gauge-Sate pairs on individual 0.25deg grid boxes
     #           npair0 (nx025,ny025), npair_s (nx025,ny025),
     #           rgage0 (nmax0,nx025,ny025), rsate0 (nmax0,nx025,ny025),
c            3.  Gauge-sate pairs over a space/time window
     #           rgage1 (nmax), rsate1 (nmax), kpair2 (nmax),
c            4.  correction table
     #           nn3 (nclss), rgag3 (nclss), rsat3 (nclss),
     #           nn4 (nclss), rgag4 (nclss), rsat4 (nclss),
     #           rat4 (nclss),
c            5.  Sate correction
     #           sate9 (nx025,ny025)
c
c     #######################################################################
c     ###                                                                 ###
c     ###  1. to collect the matching gauge and CMORPH data pairs         ###
c     ###                                                                 ###
c     #######################################################################
c
      write  (6,*)  '    started bias correction procedures !!'
c     write  (6,*)  '  kjd = ',  kjd
c
      do 1003 jj = 1, ny025  
      do 1003 ii = 1, nx025  
        npair0  (ii,jj) = 0
        npair_s (ii,jj) = 0
        sate9   (ii,jj) = rsate (ii,jj)

        do 1002 kk = 1, nmax0
          rgage0 (kk,ii,jj) = 9999.9
          rsate0 (kk,ii,jj) = 9999.9
 1002   continue
 1003 continue 

      do 1005 kdd = 1, nday_file
        kinp1 = (kdd-1)*3 + 1
        kinp2 = (kdd-1)*3 + 2
        kinp3 = (kdd-1)*3 + 3
        read  (40,rec=kinp1)  rain
        read  (40,rec=kinp2)  samp
        read  (40,rec=kinp3)  sate
c       write (6, *) kdd, kinp1, kinp2, kinp3
c       write (6, *) rain(586, 66), samp (586, 66), sate (586,66)

        do 1004 jj = 1, ny025
        do 1004 ii = 1, nx025
           if (mask025(ii,jj).eq.0) goto 1004 
             rn = rain (ii,jj)
             gn = samp (ii,jj)
             ss = sate (ii,jj)
             rd = abs (rn-ss)
             if  (gn.ge.1.0.and.
     #            rn.ge.0.0.and.
     #            ss.ge.0.0)  then
              if (ss.gt.0.0) then 
                rt = rn / ss
              else
                rt = 0.
              end if
              if (rn.gt.0.0.or.ss.gt.0.0) then
               if (rd.le.150.0.or.rt.lt.20.) then
                 npair0 (ii,jj)  = npair0 (ii,jj)  + 1
                 kk = npair0 (ii,jj)
                if (ss.gt.0.0) then
                 npair_s (ii,jj) = npair_s (ii,jj) + 1
                end if
                if  (kk.le.nmax0)  then
                  rgage0 (kk,ii,jj) = rn
                  rsate0 (kk,ii,jj) = ss
                else
                  write  (6,*)  'too many pairs :', kjd,jj,ii,kk
                  go to 1006
                end  if
               end  if
              end  if
             end  if
 1004   continue
 1005 continue

 1006 continue
      write  (6,*)  '     finished reading data pairs :', 
     #                    npair0 (586,66), npair_s (586,66) 
c 
c
c     #######################################################################
c     ###                                                                 ###
c     ###  2. to define pairs over a spatial/temporal window              ###
c     ###                                                                 ###
c     #######################################################################
c
      do 2099 jj = 1, ny025
      do 2099 ii = 1, nx025
      if(mask025(ii,jj).eq.0.or.rsate(ii,jj).eq.0.0) goto 2099
c         *****  over ocean   *****

c         *****  over land  *****
        ngrd_window0   = ngrd_window
 2002   npair1   = 0
        npair_ss = 0
        ds0 = float (ngrd_window0)

        do 2003 kk = 1, nmax
          rgage1 (kk) = 9999.9
          rsate1 (kk) = 9999.9
 2003   continue

        j1 = jj - ngrd_window0
        j2 = jj + ngrd_window0
        i1 = ii - ngrd_window0
        i2 = ii + ngrd_window0

        if (j1.lt.1 ) j1 = 1
        if (j2.gt.ny025 ) j2 = ny025

        do 2005 js = j1, j2
        do 2005 is = i1, i2

        di = abs(is-ii)
        dj = abs(js-jj)
        ds = sqrt (di*di + dj*dj)

        j = js
        if (is.ge.1.and.is.le.nx025) then
          i = is
        else if (is.lt.1) then
          i = is + nx025
        else
          i = is - nx025
        end if

        if  (i.ge.1.and.i.le.nx025.and.
     #       j.ge.1.and.j.le.ny025)  then
         if (ds.le.ds0) then
          if (npair0(i,j).gt.0.and.mask025(i,j).eq.1) then
             npair_ss = npair_ss + npair_s (i,j) 
            do 2004 kk = 1, npair0 (i,j)
              npair1 = npair1 + 1
              ll     = npair1
              if  (ll.le.nmax)  then
                rgage1 (ll) = rgage0 (kk,i,j)
                rsate1 (ll) = rsate0 (kk,i,j)
              else
                write  (6,*)  'too many pairs :',
     #          kjd, jj,ii, npair1, ds0, ds, npair1, npair_ss
                go to 2006
              end  if
 2004       continue
          end  if
         end  if
        end  if
 2005  continue

       if (npair1.lt.npair_min.or.
     #     npair_ss.lt.npair_min_s) then
         ngrd_window0 = ngrd_window0 + 1
         if (ngrd_window0.le.40) then
          go  to  2002
         else
          goto 2099
         end if
       end if

 2006 continue
c     write  (6,*)  '          finished assembling pairs !!'
c
c     #######################################################################
c     ###                                                                 ###
c     ###  3. to make a RT Bias correction table                          ###
c     ###                                                                 ###
c     #######################################################################
c
c
c      7.1  to sort the data
c
          call  qsortchr_PPX  ( rgage1, npair1, kpair2 )
          call  qsortchr_PPX  ( rsate1, npair1, kpair2 )
c
c      7.2  to group data into 100 classes
c
        do 7202 kcls = 1, nclss
          if (kcls.eq.1) then
            k1 = 1
          else
            k1 = nint (npair1*0.01*(kcls-1)) + 1
          end if
          if (kcls.eq.nclss) then
            k2 = npair1
          else
            k2 = nint (npair1*0.01*kcls)
          end if

          nn3   (kcls) = 0
          rgag3 (kcls) = 0.0
          rsat3 (kcls) = 0.0
          do 7201 kk = k1, k2
            if (rgage1(kk).lt.9999.0.and.
     #          rsate1(kk).lt.9999.0) then
              nn3   (kcls) = nn3   (kcls) + 1
              rgag3 (kcls) = rgag3 (kcls) + rgage1 (kk)
              rsat3 (kcls) = rsat3 (kcls) + rsate1 (kk)
            end if
 7201     continue
 7202   continue
c
c       7.3  to include data from beibhoring classes
c
        do 7302 kcls = 1, nclss
          kc1 = kcls - 2
          kc2 = kc1  + 4
          if (kc1.lt.1) then
            kc1 = 1
            kc2 = kc1 + 4
          end if
          if (kc2.gt.nclss) then
            kc1 = nclss - 4
            kc2 = nclss
          end if

          nn4   (kcls) = 0
          rgag4 (kcls) = 0.0
          rsat4 (kcls) = 0.0
          do 7301 kc = kc1, kc2
            if (nn3 (kc).gt.0.0) then 
            nn4   (kcls) = nn4   (kcls) + nn3   (kc)
            rgag4 (kcls) = rgag4 (kcls) + rgag3 (kc)
            rsat4 (kcls) = rsat4 (kcls) + rsat3 (kc)
            endif
 7301     continue
 7302   continue
c
c       7.4  to define mean precip and ratio for each class
c
        do 7401 kcls = 1, nclss
          if (nn3(kcls).gt.0) then
            rgag3 (kcls) = rgag3 (kcls) / nn3 (kcls)
            rsat3 (kcls) = rsat3 (kcls) / nn3 (kcls)
          else
            rgag3 (kcls) = -999.0
            rsat3 (kcls) = -999.0
          end if

          if (nn4(kcls).gt.0) then
            rgag4 (kcls) = rgag4 (kcls) / nn4 (kcls)
            rsat4 (kcls) = rsat4 (kcls) / nn4 (kcls)
            if (rsat4(kcls).gt.0.0) then
              rat4 (kcls) = rgag4 (kcls) / rsat4 (kcls)
            else
              rat4 (kcls) = 0.0
            end if
          else
            rgag4 (kcls) = -999.0
            rsat4 (kcls) = -999.0
            rat4  (kcls) = 1.0
          end if
 7401   continue

c       write(6, *) 'BEFORE nn3:', ii, jj
c       write(6, '(10i8)') (nn3(k), k=1, nclss)
c       write(6, *) 'nn4:', ii, jj
c       write(6, '(10i8)') (nn4(k), k=1, nclss)
c       write(6, *) 'rsat3:', ii, jj
c       write(6, '(10f8.3)') (rsat3(k), k=1, nclss)
c       write(6, *) 'rsat4:', ii, jj
c       write(6, '(10f8.3)') (rsat4(k), k=1, nclss)
c       write(6, *) 'rgag3:', ii, jj
c       write(6, '(10f8.3)') (rgag3(k), k=1, nclss)
c       write(6, *) 'rgag4:', ii, jj
c       write(6, '(10f8.3)') (rgag4(k), k=1, nclss)
c       write(6, *) 'rat4:', ii, jj
c       write(6, '(10f8.3)') (rat4(k), k=1, nclss)

        do 7402 kcls = 1, nclss
          if (rsat4(kcls).ge.0.0.and.rat4 (kcls).ge.0.0) then
            rmax = a / (rsat4(kcls) + b)
            if (rmax.lt.1.5)  then
              rmax = 1.5
            else if  (rmax.gt.5.0)  then
              rmax = 5.0
            end if

            if (rat4 (kcls).gt.rmax)  then
              rat4 (kcls) = rmax
            end if
          endif
 7402   continue

c       write(6, *) 'AFTER rat4:', ii, jj
c       write(6, '(10f8.3)') (rat4(kcls), kcls=1, nclss)
c
c       to define the forceful adjust factor 
        nn = 0 
        sg = 0.0 
        ss = 0.0 
        do 2312 kcls = 1, nclss 
          if  (rsat4(kcls).ge.0.0.and. 
     #         rgag4(kcls).ge.0.0.and. 
     #         rat4(kcls).ge.0.0)  then 
            nn = nn + 1  
            sg = sg + rgag4 (kcls) 
            ss = ss + rsat4 (kcls) * rat4 (kcls) 
          end  if  
 2312   continue 

        if  (nn.ge.50.and.ss.gt.0.0)  then 
          fct (ii,jj) = sg / ss 
        else 
          fct (ii,jj) = 1.0 
        end  if  
c       write (*,*) 'fct=', fct (ii,jj)
c
c     #######################################################################
c     ###                                                                 ###
c     ###  5. to correct the bias thru CDF matching                       ###
c     ###                                                                 ###
c     #######################################################################
c
        rat = -999.0
        if (rsate(ii,jj).le.rsat4(1)) then
          rat = rat4 (1)
        else if (rsate(ii,jj).ge.rsat4(nclss)) then
          rat = rat4  (nclss)
        else
          kcls = 1
          do 8002 kk = 1, nclss-1
            if (rsate(ii,jj).ge.rsat4(kk).and.
     #          rsate(ii,jj).lt.rsat4(kk+1)) then
              kcls = kk
              go  to  8003
            end  if
 8002     continue
 8003     continue

          rt0 = rat4(kcls)
          rt1 = rat4(kcls+1)
          rn0 = rsat4(kcls)
          rn1 = rsat4(kcls+1)
          rn  = rsate(ii,jj)
          if  (rt0.ge.0.0.and.
     #         rt1.ge.0.0.and.
     #         rn0.ge.0.0.and.
     #         rn1.ge.0.0)  then
            if  (abs(rn1 - rn0).lt.1.e-6)  then
              rat = rt0
            else
              rat = rt0 + (rt1-rt0)*(rn-rn0)/(rn1-rn0)
            end  if
          else
            rat = 1.0
          end  if
        end if
c       write(6, *) 'rat=', rat

        if  (rat.lt.0.0) then
          rat = 1.0
        else if  (rat.gt.0.0) then
          rmax = a / (rsate(ii,jj) + b)
          if  (rmax.lt.1.5)  then
            rmax = 1.5
          else  if  (rmax.gt.5.0)  then
            rmax = 5.0
          end  if
          if  (rat.gt.rmax)  then
            rat = rmax
          end  if
        end  if

        ratio (ii,jj) = rat
        sate9 (ii,jj) = rat * rsate (ii,jj) 
c       write (*,*) 'rat=', rat,rsate(ii,jj),rgage (ii,jj),sate9(ii,jj)

        if  (fct(ii,jj).gt.0.0)  then
          if  (fct(ii,jj).lt.0.95)  then
            ff = 0.95
          else  if  (fct(ii,jj).le.1.05)  then
            ff = fct (ii,jj)
          else
            ff = 1.05
          end  if
          sate9 (ii,jj) = ff * sate9 (ii,jj)
        end  if
c       write (*,*) 'ff=', ff, rsate (ii,jj), sate9 (ii,jj)
 2099 continue

c
c     #######################################################################
c     ###                                                                 ###
c     ###  6. to output the results                                       ###
c     ###                                                                 ###
c     #######################################################################
c
      kstatus = 0

      write (50,rec=1)   sate9

      print*,'      sate, ratio, gage, sate9, fct = ',
     # rsate(586,66),ratio(586,66),rgage(586,66),sate9(586,66),
     # fct (586,66)

      close (40)
      close (50)

      return 
      end

c
c
c     #######################################################################
c     ###                                                                 ###
c     ###  Subroutine  :                                                  ###
c     ###                                                                 ###
c     ###                 name:       qsortchr_PPX(arr,n, karr)           ###
c     ###                                                                 ###
c     ###                 function:   to sort an array in accesending     ###
c     ###                             orider                              ###
c     ###                                                                 ###
c     ###                 inputs:                                         ###
c     ###                             arr  ==   the array to be sorted    ###
c     ###                             n    ==   dimension of the array    ###
c     ###                                                                 ###
c     ###                 output:                                         ###
c     ###                             karr ==   position of the sorted    ###
c     ###                             arr  ==   the sorted array          ###
c     ###                                                                 ###
c     #######################################################################
c
C  qsortReal: sort real array elements
c (Numerical Recioes in FORTRAN 77.  2nd edition. Page 324-325.)
C
      subroutine qsortchr_PPX(arr,n, karr)
c
      integer*4 n,M,NSTACK
      real*4 arr(n)
      parameter(M=7,NSTACK=1000000)
      integer*4 i,ir,j,jstack,k,l,istack(NSTACK)
      real*4 a,temp
      integer*4  karr (n)

      do i = 1, n
        karr (i) = i
      end  do

      jstack=0
      L=1
      ir=n


   1  if(ir-L.lt.M) then
       do j=L+1,ir
        a=arr(j)
        ka = karr(j)
        do i=j-1,L,-1
         if(arr(i).lt.a) goto 2
         arr(i+1)=arr(i)
         karr(i+1) = karr(i)
        enddo
        i=L-1
   2    arr(i+1)=a
        karr(i+1) = ka
       enddo
       if(jstack.eq.0) return
       ir=istack(jstack)
       L=istack(jstack-1)
       jstack=jstack-2
      else
       k=(L+ir)/2
       temp=arr(k)
       ktemp = karr(k)
       arr(k)=arr(L+1)
       karr(k)=karr(L+1)
       arr(L+1)=temp
       karr(L+1)=ktemp
       if(arr(L).gt.arr(ir)) then
        temp=arr(L)
        ktemp=karr(L)
        arr(L)=arr(ir)
        karr(L) = karr(ir)
        arr(ir)=temp
        karr(ir)=ktemp
       endif
       if(arr(L+1).gt.arr(ir)) then
        temp=arr(L+1)
        ktemp = karr(L+1)
        arr(L+1)=arr(ir)
        karr(L+1) = karr(ir)
        arr(ir)=temp
        karr(ir) = ktemp
       endif
       if(arr(L).gt.arr(L+1)) then
        temp=arr(L)
        ktemp = karr(L)
        arr(L)=arr(L+1)
        karr(L) = karr(L+1)
        arr(L+1)=temp
        karr(L+1)=ktemp
       endif
       i=L+1
       j=ir
       a=arr(L+1)
       ka = karr(L+1)
   3   continue
       i=i+1
       if(arr(i).lt.a) goto 3
   4   continue
       j=j-1
       if(arr(j).gt.a) goto 4
       if(j.lt.i) goto 5
       temp=arr(i)
       ktemp = karr(i)
       arr(i)=arr(j)
       karr(i) = karr(j)
       arr(j)=temp
       karr(j)= ktemp
       goto 3
   5   arr(L+1)=arr(j)
       karr(L+1)=karr(j)
       arr(j)=a
       karr(j) = ka
       jstack=jstack+2
c      if(jstack.gt.NSTACK) pause 'NSTACK too small in sort'
       if(jstack.gt.NSTACK) stop 'NSTACK too small in sort'
       if(ir-i+1.ge.j-L) then
        istack(jstack)=ir
        istack(jstack-1)=i
        ir=j-1
       else
        istack(jstack)=j-1
        istack(jstack-1)=L
        L=i
       endif
      endif
      goto 1
      return
      end
