c@***h* CMORPH-CDR/l4.CMORPH_V1.0_CRT.get_FCT_LND_0.25deg-DLY.f
c
c=======================================================================
c PROGRAM
c   l4.CMORPH_V1.0_CRT.get_FCT_LND_0.25deg-DLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PUPROSE
c   to generate 0.25deg-DLY adjust factors over land using 7 days of
c   0.25deg-DLY_EOD raw and land-adjusted CMORPH data centering at
c   target date
c
c-----------------------------------------------------------------------
c USAGE:
c   l4.CMORPH_V1.0_CRT.get_FCT_LND_0.25deg-DLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES:
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx          : 0.25deg land mask
c   CMORPH_V1.0_RAW_0.25deg-DLY_EOD_<yyyymmdd> : 0.25deg-DLY_EOD raw
c                                                CMORPH data
c   CMORPH_ADJ_LND_0.25deg-DLY_EOD_<yyyymmdd>  : 0.25deg-DLY_EOD land-
c                                                adjusted CMORPH data
c
c OUTPUT FILES:
c   FCT_LND_0.25deg-DLY_<yyyymmdd> : 0.25deg-DLY adjust factors over land
c
c-----------------------------------------------------------------------
c INPUT VARIABLES:
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES:
c   NX             : X-dimension (0-360) in 0.25deg grid
c   NY             : Y-dimension (60S-60N) in 0.25deg grid
c   N3Y            : number of days in 3 years
c   NGRD           : number of grids in spatial smoothing
c   WIN            : half of processing window
c   ND             : number of days in processing window
c   ---------------:------------------------------------
c   yyyymmdd       : target date
c   f_msk          : 0.25deg land mask file name
c   f_raw          : 0.25deg-DLY_EOD raw CMORPH data file name
c   f_adj          : 0.25deg-DLY_EOD land-adjusted CMORPH data file name
c   f_fct          : file name of 0.25deg-DLY adjust factor over land
c   yyyy,mm,dd     : target date
c   days()         : number of days in each month
c   y,m,d          : loop counters for years, months, days
c   td             : index in yyyy-mm-dd tables for target date
c   k              : counter for days in yyyy-mm-dd tables
c   k1,k2          : index boundaries for 7-day yyyy-mm-dd tables
c   y0(),m0(),d0() : 3-year tables of yyyy-mm-dd
c   y1(),m1(),d1() : yyyy-mm-dd tables for processing window
c   ii, i          : loop counters in X-direction
c   i1, i2         : X-boundaries for spatial smoothing
c   is             : X-index of working grid box
c   jj, j          : loop counters in Y-direction
c   j1, j2         : Y-boundaries for spatial smoothing
c   js             : Y-index of working grid box
c   dis2           : distance^2 between working and target grid boxes
c   dis2max        : maximum dis2
c   mask()         : 0.25deg land mask data
c   raw()          : 0.25deg-DLY_EOD raw CMORPH data
c   adj()          : 0.25deg-DLY_EOD land-adjusted CMORPH data
c   fct()          : 0.25deg-DLY adjust factors over land
c   sw()           : sum of weight over processing window
c   sr()           : sum of raw data over processing window
c   sa()           : sum of land-adjusted data over processing window
c   ww             : weight
c   ssw            : sum of sw() over smoothing window
c   ssr            : sum of sr() over smoothing window
c   ssa            : sum of sa() over smoothing window
c   
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_0.25deg_dly_land.lnx.f'
c
c  step 4 of 4 to generate 0.25deg-DLY adjust factors over land
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program l4_CMORP_V10_CRT_get_FCT_LND_025deg_DLY
      implicit none

      integer NX, NY, N3Y, NGRD, WIN, ND
      parameter( NX=1440, NY=480, N3Y=1096, NGRD=2, WIN=3, ND=2*WIN+1 )

      character yyyymmdd*8, f_msk*42, f_raw*48, f_adj*47, f_fct*36
      integer yyyy, mm, dd, days(12), y, m, d, td, k, k1, k2
      integer y0(N3Y), m0(N3Y), d0(N3Y), y1(ND), m1(ND), d1(ND)
      integer ii, jj, i, i1, i2, is, j, j1, j2, js, dis2, dis2max
      real mask(NX,NY), raw(NX,NY), adj(NX,NY), fct(NX,NY)
      real sw(NX,NY), sr(NX,NY), sa(NX,NY), ww, ssw, ssr, ssa

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c create 3-year tables of yyyy-mm-dd
c-----------------------------------------------------------------------
      do k=1, N3Y
        y0(k)=-999
        m0(k)=-999
        d0(k)=-999
      enddo
      k=0
      do y=yyyy-1, yyyy+1
        days(2)=28
        if( mod(y,4).eq.0 )   days(2)=29
        if( mod(y,100).eq.0 ) days(2)=28
        if( mod(y,400).eq.0 ) days(2)=29
        do m=1, 12
          do d=1, days(m)
            k=k+1
            y0(k)=y
            m0(k)=m
            d0(k)=d
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c find index for target date
c-----------------------------------------------------------------------
      td=-999
      do k=1, N3Y
        if( y0(k).eq.yyyy.and.m0(k).eq.mm.and.d0(k).eq.dd ) then
          td=k
          exit
        endif
      enddo

c-----------------------------------------------------------------------
c get yyyy-mm-dd tables for processing window
c-----------------------------------------------------------------------
      if( yyyy.eq.1998.and.mm.eq.1.and.dd.le.WIN ) then
        k1=365+1
        k2=365+ND
      else
        k1=td-WIN
        k2=td+WIN
      endif
      d=0
      do k=k1, k2
        d=d+1
        y1(d)=y0(k)
        m1(d)=m0(k)
        d1(d)=d0(k)
      enddo

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|12345678<--
      f_msk='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_raw='../work/CMORPH_V1.0_RAW_0.25deg-DLY_EOD_yyyymmdd'
      f_adj='../work/CMORPH_ADJ_LND_0.25deg-DLY_EOD_yyyymmdd'
      f_fct='../work/FCT_LND_0.25deg-DLY_yyyymmdd'

      if( iargc().ge.2 ) then
        f_raw(4:7)='redo'
        f_adj(4:7)='redo'
        f_fct(4:7)='redo'
      endif

      write(f_fct(29:36), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c read in 0.25deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_msk
      read(10, rec=1) mask
      close(10)

c-----------------------------------------------------------------------
c initialize arrays
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          sw(ii,jj)=0.0
          sr(ii,jj)=0.0
          sa(ii,jj)=0.0
        enddo
      enddo

c-----------------------------------------------------------------------
c read in and sum up raw and adjusted data for processing window
c-----------------------------------------------------------------------
      do d=1, ND

        write(f_raw(41:48), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

        write(f_adj(40:47), '(i4.4,2i2.2)') y1(d), m1(d), d1(d)

        open(11, file=f_raw, status='old',
     1      access='direct', recl=NX*NY*4)
        open(12, file=f_adj, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_raw
        write(*, *) ' <--  ', f_adj
        read(11, rec=1) raw
        read(12, rec=1) adj
        close(11)
        close(12)

        ww=1.0
        do jj=1, NY
          do ii=1, NX
            if( mask(ii,jj).ne.0.0.and.
     1          raw(ii,jj).ge.0.0.and.
     2          adj(ii,jj).ge.0.0 ) then
              sw(ii,jj)=sw(ii,jj)+ww
              sr(ii,jj)=sr(ii,jj)+ww*raw(ii,jj)
              sa(ii,jj)=sa(ii,jj)+ww*adj(ii,jj)
            endif
          enddo
        enddo

      enddo

c-----------------------------------------------------------------------
c perform spatial smoothing
c-----------------------------------------------------------------------
      dis2max=NGRD*NGRD

      do jj=1, NY
        do ii=1, NX

          if( mask(ii,jj).eq.0.0 ) then
            fct(ii,jj)=-999.0
          else
            j1=jj-NGRD
            if( j1.lt.1 ) j1=1
            j2=jj+NGRD
            if( j2.gt.NY ) j2=NY
            i1=ii-NGRD
            i2=ii+NGRD
            ssw=0.0
            ssr=0.0
            ssa=0.0
            do j=j1, j2
              do i=i1, i2
                dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                if( dis2.le.dis2max ) then
                  ww=1.0
                  js=j
                  if( i.lt.1 ) then
                    is=i+NX
                  else if( i.ge.1.and.i.le.NX ) then
                    is=i
                  else
                    is=i-NX
                  endif
                  ssw=ssw+sw(is,js)
                  ssr=ssr+sr(is,js)
                  ssa=ssa+sa(is,js)
                endif
              enddo
            enddo
            if( ssw.eq.0.0 ) then
              fct(ii,jj)=-999.0
            else
              if( ssr.eq.0.0 ) then
                fct(ii,jj)=1.0
              else
                fct(ii,jj)=ssa/ssr
              endif
            endif
          endif

        enddo
      enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
      open(20, file=f_fct, access='direct', recl=NX*NY*4)
      write(20, rec=1) fct
      close(20)
      write(*, *) '  ==> ', f_fct

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
