c@***h* CMORPH-CDR/o1.CMORPH_V1.0_CRT.get_RAW_0.25deg-PEN.RAW_2.50deg-PEN.f
c
c=======================================================================
c PROGRAM
c   o1.CMORPH_V1.0_CRT.get_RAW_0.25deg-PEN.RAW_2.50deg-PEN.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to calculate pentad CMORPH (0.25deg and 2.50deg) from 0.25deg-DLY
c   values ending at 00Z
c
c-----------------------------------------------------------------------
c USAGE
c   o1.CMORPH_V1.0_CRT.get_RAW_0.25deg-PEN.RAW_2.50deg-PEN.x <yyyypp> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx          : 0.25deg land mask
c   CMORPH_V1.0_LAND_MASK_2.50deg.lnx          : 2.50deg land mask
c   CMORPH_V1.0_RAW_0.25deg-DLY_00Z_<yyyymmdd> : 0.25deg-DLY CMORPH data 
c
c OUTPUT FILES
c   CMORPH_0.25deg-PEN_<yyyypp> : 0.25deg-PEN CMORPH data
c   CMORPH_2.50deg-PEN_<yyyypp> : 2.50deg-PEN CMORPH data
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyypp : target pentad
c   redo   : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX        : X-dimension (0-360) in 0.25deg grid
c   NY        : Y-dimension (60S-60N) in 0.25deg grid
c   NX2       : X-dimension (0-360) in 2.50deg grid
c   NY2       : Y-dimension (60S-60N) in 2.50deg grid
c   NP        : number of pentads in a year
c   ----------:----------------------------
c   yyyypp    : target pentad
c   f_mk0     : 0.25deg land mask file name
c   f_mk2     : 2.50deg land mask file name
c   f_025     : 0.25deg-PEN data file name
c   f_250     : 2.50deg-PEN data file name
c   f_dly     : 0.25deg-DLY_00Z data file name
c   yyyy,pp   : target pentad
c   leap      : leap-year flag
c   mmdd()    : month-day for each pentad
c   d         : loop counter for days
c   nn()      : counts of valid 0.25deg data points
c   nn2()     : counts of valid 2.50deg data points
c   ii, i     : loop counters in X-direction
c   jj, j     : loop counters in Y-direction
c   i1, i2    : X-boundaries of 2.50deg grid box in 0.25deg grid
c   j1, j2    : Y-boundaries of 2.50deg grid box in 0.25deg grid
c   mask()    : 0.25deg land mask
c   rain()    : 0.25deg-DLY data
c   ss()      : 0.25deg-PEN data
c   lat       : latitude of grid box
c   w         : weight associate dwith latitude
c   mask2()   : 2.50deg land mask
c   ss2()     : 2.50deg-PEN data
c   sw2()     : sum of weights
c   dly_there : check if 0.25deg-DLY data file exists
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'CMORPH_def_2.50deg_PEN.lnx.f'
c
c   step 1 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o1_CMORPH_V10_CRT_get_RAW_025deg_PEN_RAW_250deg_PEN
      implicit none

      integer NX, NY, NX2, NY2, NP
      parameter( NX=1440, NY=480, NX2=144, NY2=48, NP=73 )

      character yyyypp*6, f_mk0*42, f_mk2*42
      character f_025*33, f_250*33, f_dly*48
      integer yyyy, pp, leap, mmdd(5,NP), d
      integer nn(NX,NY), nn2(NX2,NY2), ii, jj, i, i1, i2, j, j1, j2
      real mask(NX,NY), rain(NX,NY), ss(NX,NY), lat, w
      real mask2(NX2,NY2), ss2(NX2,NY2), sw2(NX2,NY2)
      logical dly_there

      data mmdd /
     1     0101, 0102, 0103, 0104, 0105,  0106, 0107, 0108, 0109, 0110,
     1     0111, 0112, 0113, 0114, 0115,  0116, 0117, 0118, 0119, 0120,
     1     0121, 0122, 0123, 0124, 0125,  0126, 0127, 0128, 0129, 0130,
     2     0131, 0201, 0202, 0203, 0204,  0205, 0206, 0207, 0208, 0209,
     2     0210, 0211, 0212, 0213, 0214,  0215, 0216, 0217, 0218, 0219,
     2     0220, 0221, 0222, 0223, 0224,  0225, 0226, 0227, 0228, 0301,
     3     0302, 0303, 0304, 0305, 0306,  0307, 0308, 0309, 0310, 0311,
     3     0312, 0313, 0314, 0315, 0316,  0317, 0318, 0319, 0320, 0321,
     3     0322, 0323, 0324, 0325, 0326,  0327, 0328, 0329, 0330, 0331,
     4     0401, 0402, 0403, 0404, 0405,  0406, 0407, 0408, 0409, 0410,
     4     0411, 0412, 0413, 0414, 0415,  0416, 0417, 0418, 0419, 0420,
     4     0421, 0422, 0423, 0424, 0425,  0426, 0427, 0428, 0429, 0430,
     5     0501, 0502, 0503, 0504, 0505,  0506, 0507, 0508, 0509, 0510,
     5     0511, 0512, 0513, 0514, 0515,  0516, 0517, 0518, 0519, 0520,
     5     0521, 0522, 0523, 0524, 0525,  0526, 0527, 0528, 0529, 0530,
     6     0531, 0601, 0602, 0603, 0604,  0605, 0606, 0607, 0608, 0609,
     6     0610, 0611, 0612, 0613, 0614,  0615, 0616, 0617, 0618, 0619,
     6     0620, 0621, 0622, 0623, 0624,  0625, 0626, 0627, 0628, 0629,
     7     0630, 0701, 0702, 0703, 0704,  0705, 0706, 0707, 0708, 0709,
     7     0710, 0711, 0712, 0713, 0714,  0715, 0716, 0717, 0718, 0719,
     7     0720, 0721, 0722, 0723, 0724,  0725, 0726, 0727, 0728, 0729,
     8     0730, 0731, 0801, 0802, 0803,  0804, 0805, 0806, 0807, 0808,
     8     0809, 0810, 0811, 0812, 0813,  0814, 0815, 0816, 0817, 0818,
     8     0819, 0820, 0821, 0822, 0823,  0824, 0825, 0826, 0827, 0828,
     9     0829, 0830, 0831, 0901, 0902,  0903, 0904, 0905, 0906, 0907,
     9     0908, 0909, 0910, 0911, 0912,  0913, 0914, 0915, 0916, 0917,
     9     0918, 0919, 0920, 0921, 0922,  0923, 0924, 0925, 0926, 0927,
     a     0928, 0929, 0930, 1001, 1002,  1003, 1004, 1005, 1006, 1007,
     a     1008, 1009, 1010, 1011, 1012,  1013, 1014, 1015, 1016, 1017,
     a     1018, 1019, 1020, 1021, 1022,  1023, 1024, 1025, 1026, 1027,
     b     1028, 1029, 1030, 1031, 1101,  1102, 1103, 1104, 1105, 1106,
     b     1107, 1108, 1109, 1110, 1111,  1112, 1113, 1114, 1115, 1116,
     b     1117, 1118, 1119, 1120, 1121,  1122, 1123, 1124, 1125, 1126,
     c     1127, 1128, 1129, 1130, 1201,  1202, 1203, 1204, 1205, 1206,
     c     1207, 1208, 1209, 1210, 1211,  1212, 1213, 1214, 1215, 1216,
     c     1217, 1218, 1219, 1220, 1221,  1222, 1223, 1224, 1225, 1226,
     c     1227, 1228, 1229, 1230, 1231 /

c-----------------------------------------------------------------------
c get target pentad
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '########################################'
        write(*, *) '#                                      #'
        write(*, *) '#  parameter <yyyypp> [<redo>] needed  #'
        write(*, *) '#                                      #'
        write(*, *) '########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyypp)
      read(yyyypp, '(i4.4,i2.2)') yyyy, pp

c-----------------------------------------------------------------------
c adjust for leap-year
c-----------------------------------------------------------------------
      leap=0
      if( mod(yyyy,4).eq.0 )   leap=1
      if( mod(yyyy,100).eq.0 ) leap=0
      if( mod(yyyy,400).eq.0 ) leap=1

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|12345678<--
      f_mk0='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_mk2='../table/CMORPH_V1.0_LAND_MASK_2.50deg.lnx'
      f_025='../work/CMORPH_0.25deg-PEN_yyyypp'
      f_250='../work/CMORPH_2.50deg-PEN_yyyypp'
      f_dly='../work/CMORPH_V1.0_RAW_0.25deg-DLY_00Z_yyyymmdd'

      if( iargc().ge.2 ) then
        f_025(4:7)='redo'
        f_250(4:7)='redo'
        f_dly(4:7)='redo'
      endif

      write(f_025(28:33), '(i4.4,i2.2)') yyyy, pp
      write(f_250(28:33), '(i4.4,i2.2)') yyyy, pp

c-----------------------------------------------------------------------
c read in land masks
c-----------------------------------------------------------------------
      open(10, file=f_mk0, status='old',
     1    access='direct', recl=NX*NY*4)
      open(12, file=f_mk2, status='old',
     1    access='direct', recl=NX2*NY2*4)
      write(*, *) ' <--  ', f_mk0
      write(*, *) ' <--  ', f_mk2

      read(10, rec=1) mask
      read(12, rec=1) mask2

      close(10)
      close(12)

c-----------------------------------------------------------------------
c initialize 0.25deg arrays
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          nn(ii,jj)=0
          ss(ii,jj)=0.0
        enddo
      enddo

c-----------------------------------------------------------------------
c read in and sum up 0.25deg-DLY data for 5 days
c-----------------------------------------------------------------------
      do d=1, 5

        write(f_dly(41:48), '(2i4.4)') yyyy, mmdd(d,pp)

        inquire(file=f_dly, exist=dly_there)
        if( .not.dly_there ) then
          write(*, *) '#### file missing: ', f_dly
          stop
        endif

        open(11, file=f_dly, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_dly
        read(11, rec=1) rain
        close(11)

        do jj=1, NY
          do ii=1, NX
            if( mask(ii,jj).eq.0.0.and.rain(ii,jj).ge.0.0 ) then
              nn(ii,jj)=nn(ii,jj)+1
              ss(ii,jj)=ss(ii,jj)+rain(ii,jj)
            endif
          enddo
        enddo

      enddo

c-----------------------------------------------------------------------
c add the 6th day for pentad-12 in a leap-year
c-----------------------------------------------------------------------
      if( leap.eq.1.and.pp.eq.12 ) then

        write(f_dly(41:48), '(2i4.4)') yyyy, 0229

        inquire(file=f_dly, exist=dly_there)
        if( .not.dly_there ) then
          write(*, *) '#### file missing: ', f_dly
          stop
        endif

        open(11, file=f_dly, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_dly
        read(11, rec=1) rain
        close(11)

        do jj=1, NY
          do ii=1, NX
            if( mask(ii,jj).eq.0.0.and.rain(ii,jj).ge.0.0 ) then
              nn(ii,jj)=nn(ii,jj)+1
              ss(ii,jj)=ss(ii,jj)+rain(ii,jj)
            endif
          enddo
        enddo

      endif

c-----------------------------------------------------------------------
c define 0.25deg-PEN data
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          if( nn(ii,jj).ge.3 ) then
            ss(ii,jj)=ss(ii,jj)/nn(ii,jj)
          else
            ss(ii,jj)=-999.0
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c initialize 2.50deg arrays
c-----------------------------------------------------------------------
      do jj=1, NY2
        do ii=1, NX2
          nn2(ii,jj)=0
          ss2(ii,jj)=0.0
          sw2(ii,jj)=0.0
        enddo
      enddo

c-----------------------------------------------------------------------
c define 2.50deg-PEN data from 0.25deg-PEN data
c-----------------------------------------------------------------------
      do jj=1, NY2
        do ii=1, NX2
          j1=(jj-1)*10+1
          j2=j1+9
          i1=(ii-1)*10+1
          i2=i1+9
          do j=j1, j2
            do i=i1, i2
              if( mask(i,j).eq.0.0.and.ss(i,j).ge.0.0 ) then
                lat=float(j-1)*0.25-59.875
                lat=lat*3.14159/180.0
                w=cos(lat)
                nn2(ii,jj)=nn2(ii,jj)+1
                sw2(ii,jj)=sw2(ii,jj)+w
                ss2(ii,jj)=ss2(ii,jj)+w*ss(i,j)
              endif
            enddo
          enddo
        enddo
      enddo

      do jj=1, NY2
        do ii=1, NX2
          if( mask2(ii,jj).eq.0.0.and.nn2(ii,jj).gt.10 ) then
            ss2(ii,jj)=ss2(ii,jj)/sw2(ii,jj)
          else
            ss2(ii,jj)=-999.0
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
      open(20, file=f_025, access='direct', recl=NX*NY*4)
      open(21, file=f_250, access='direct', recl=NX2*NY2*4)

      write(20, rec=1) ss
      write(21, rec=1) ss2

      close(20)
      close(21)
      write(*, *) '  ==> ', f_025
      write(*, *) '  ==> ', f_250

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
