c@***h* CMORPH-CDR/o2.CMORPH_V1.0_CRT.get_GPCP_RAW_2.50deg-PEN.f
c
c=======================================================================
c PROGRAM
c   o2.CMORPH_V1.0_CRT.get_GPCP_RAW_2.50deg-PEN.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to separate 2.50deg-PEN (90S-90N) GPCP data from yearly file to 
c   pentad files
c
c-----------------------------------------------------------------------
c USAGE
c   o2.CMORPH_V1.0_CRT.get_GPCP_RAW_2.50deg-PEN.x <yyyy> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   gpcp_pen_v2.2.lnx.<yyyy> : 2.50deg-PEN GPCP data yearly file
c
c OUTPUT FILES
c   GPCP_RAW_2.50deg-PEN_<yyyypp> : 2.50deg-PEN GPCP data pentad files
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyy : target year
c   redo : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX     : X-dimension (0-360) in 2.50deg grid
c   NY     : Y-dimension (90S-90N) in 2.50deg grid
c   NP     : number of pentad in a year
c   -------:---------------------------
c   yyyy   : target year
c   f_year : 2.50deg-PEN GPCP data yearly file name
c   f_pent : 2.50deg-PEN GPCP data pentad file name
c   gpcp() : 2.50deg-PEN GPCP data
c   pp     : loop counter for pentad
c   ii     : loop counter in X-direction
c   jj     : loop counter in Y-direction
c   cc     : count of valid data points
c   
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'separate_gpcp_2.50deg_pen.f'
c
c   step 2 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o2_CMORPH_V10_CRT_get_GPCP_RAW_250deg_PEN
      implicit none

      integer NX, NY, NP
      parameter( NX=144, NY=72, NP=73 )

      character yyyy*4, f_year*30, f_pent*35
      real gpcp(NX,NY)
      integer pp, ii, jj, cc

c-----------------------------------------------------------------------
c get target year
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '######################################'
        write(*, *) '#                                    #'
        write(*, *) '#  parameter <yyyy> [<redo>] needed  #'
        write(*, *) '#                                    #'
        write(*, *) '######################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyy)

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c------------>123456789|123456789|123456789|12345<--
      f_year='../work/gpcp_pen_v2.2.lnx.yyyy'
      f_pent='../work/GPCP_RAW_2.50deg-PEN_yyyypp'

      if( iargc().ge.2 ) then
        f_year(4:7)='redo'
        f_pent(4:7)='redo'
      endif

      write(f_year(27:30), '(a4)') yyyy

      write(f_pent(30:33), '(a4)') yyyy

c-----------------------------------------------------------------------
c open yearly file
c-----------------------------------------------------------------------
      open(10, file=f_year, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_year

c-----------------------------------------------------------------------
c repeat for each pentad
c-----------------------------------------------------------------------
      do pp=1, NP

c-----------------------------------------------------------------------
c read in 2.50deg-PEN GPCP data
c-----------------------------------------------------------------------
        read(10, rec=pp) gpcp

c-----------------------------------------------------------------------
c count valid data points
c-----------------------------------------------------------------------
        cc=0
        do jj=1, NY
          do ii=1, NX
            if( gpcp(ii,jj).ge.0.0 ) then
              cc=cc+1
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
        if( cc.gt.100 ) then
          write(f_pent(34:35), '(i2.2)') pp
          open(20, file=f_pent, access='direct', recl=NX*NY*4)
          write(20, rec=1) gpcp
          close(20)
          write(*, *) '  ==> ', f_pent
        endif

      enddo

      close(10)

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
