c@***h* CMORPH-CDR/o5.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-PEN.f
c
c=======================================================================
c PROGRAM
c   o5.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-PEN.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to spatially downscale 2.50deg-PEN adjust factors over ocean into
c   0.25deg-PEN adjust factors over ocean
c
c-----------------------------------------------------------------------
c USAGE
c   o5.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-PEN.x <yyyypp> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx : 0.25deg land mask
c   FCT_OCN_2.50deg-PEN_<yyyypp>      : 2.50deg-PEN adjust factors
c                                       over ocean
c
c OUTPUT FILES
c   FCT_OCN_0.25deg-PEN_<yyyypp> : 0.25deg-PEN adjust factors over ocean
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyypp : target pentad
c   redo   : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX      : X-dimension (0-360) in 0.25deg grid
c   NY      : Y-dimension (60S-60N) in 0.25deg grid
c   NX2     : X-dimension (0-360) in 2.50deg grid
c   NY2     : Y-dimension (60S-60N) in 2.50deg grid
c   NGRD    : number of grid boxes in spatial smoothing
c   --------:------------------------------------------
c   yyyypp  : target pentadc
c   f_msk   : 0.25deg land mask file name
c   f_250   : file name of 2.50deg-PEN adjust factors over ocean
c   f_025   : file name of 0.25deg-PEN adjust factors over ocean
c   yyyy,pp : target pentad
c   ii,i    : loop counters in X-direction
c   i1,i2   : X-boundaries in smoothing/searching
c   is      : X-index of working grid box
c   jj,j    : loop counters in Y-direction
c   j1,j2   : Y-boundaries in smoothing/searching
c   js      : Y-index of working grid box
c   mask()  : 0.25deg land mask
c   fct2()  : 2.50deg-PEN adjust factors over ocean
c   fct0()  : 0.25deg-PEN adjust factors over ocean
c   fct()   : working array for 0.25deg-PEN adjust factors
c   ct      : count of valid data points
c   dis     : searching distance
c   dis2    : distance^2 between working and target grid boxes
c   dis2max : maximum dis2
c   wt      : weight
c   sw      : sum of weights
c   sf      : sum of adjust factors
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_0.25deg_pen_ocean.lnx.f'
c
c   step 5 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o5_CMORPH_V10_CRT_get_FCT_OCN_025deg_PEN
      implicit none

      integer NX, NY, NX2, NY2, NGRD
      parameter( NX=1440, NY=480, NX2=144, NY2=48, NGRD=5 )

      character yyyypp*6, f_msk*42, f_250*34, f_025*34
      integer yyyy, pp, ii, jj, i, i1, i2, is, j, j1, j2, js
      real mask(NX,NY), fct2(NX2,NY2), fct0(NX,NY), fct(NX,NY)
      integer ct, dis, dis2, dis2max
      real wt, sw, sf
      
c-----------------------------------------------------------------------
c get target pentad
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '########################################'
        write(*, *) '#                                      #'
        write(*, *) '#  parameter <yyyypp> [<redo>] needed  #'
        write(*, *) '#                                      #'
        write(*, *) '########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyypp)
      read(yyyypp, '(i4.4,i2.2)') yyyy, pp

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|12<--
      f_msk='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_250='../work/FCT_OCN_2.50deg-PEN_yyyypp'
      f_025='../work/FCT_OCN_0.25deg-PEN_yyyypp'

      if( iargc().ge.2 ) then
        f_250(4:7)='redo'
        f_025(4:7)='redo'
      endif

      write(f_250(29:34), '(i4.4,i2.2)') yyyy,pp
      write(f_025(29:34), '(i4.4,i2.2)') yyyy,pp

c-----------------------------------------------------------------------
c read in 0.25deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_msk
      read(10, rec=1) mask
      close(10)

c-----------------------------------------------------------------------
c read in 2.50deg-PEN adjust factors over ocean
c-----------------------------------------------------------------------
      open(11, file=f_250, status='old',
     1    access='direct', recl=NX2*NY2*4)
      write(*, *) ' <--  ', f_250
      read(11, rec=1) fct2
      close(11)

c-----------------------------------------------------------------------
c disaggregate into 0.25deg grid
c-----------------------------------------------------------------------
      do jj=1, NY2
        do ii=1, NX2
          j1=(jj-1)*10+1
          j2=j1+9
          i1=(ii-1)*10+1
          i2=i1+9
          do j=j1, j2
            do i=i1, i2
              fct0(i,j)=fct2(ii,jj)
            enddo
          enddo
        enddo
      enddo

c-----------------------------------------------------------------------
c smooth out seams
c-----------------------------------------------------------------------
      dis2max=NGRD*NGRD
      do jj=1, NY
        do ii=1, NX
          if( fct0(ii,jj).lt.0.0 ) then
            fct(ii,jj)=-999.0
          else
            j1=jj-NGRD
            j2=jj+NGRD
            if( j1.lt.1 ) j1=1
            if( j2.gt.NY ) j2=NY
            i1=ii-NGRD
            i2=ii+NGRD
            ct=0
            sw=0.0
            sf=0.0
            do j=j1, j2
              do i=i1, i2
                dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                if( dis2.le.dis2max ) then
                  js=j
                  if( i.lt.1 ) then
                    is=i+NX
                  elseif( i.ge.1.and.i.le.NX ) then
                    is=i
                  else
                    is=i-NX
                  endif
                  if( fct0(is,js).ge.0.0 ) then
                    wt=1.0/(sqrt(float(dis2))+1.0)
                    ct=ct+1
                    sw=sw+wt
                    sf=sf+wt*fct0(is,js)
                  endif
                endif
              enddo
            enddo
            if( ct.gt.0.and.sw.gt.0.0 ) then
              fct(ii,jj)=sf/sw
            else
              fct(ii,jj)=-999.0
            endif
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c fill in gaps
c-----------------------------------------------------------------------
      do jj=1, NY
        do ii=1, NX
          fct0(ii,jj)=fct(ii,jj)
        enddo
      enddo
      do jj=1, NY
        do ii=1, NX
          if( mask(ii,jj).ne.0.0 ) then
            fct0(ii,jj)=-999.0
          else
            dis=0
 100        dis2max=dis*dis
            j1=jj-dis
            if( j1.lt.1 ) j1=1
            j2=jj+dis
            if( j2.gt.NY ) j2=NY
            i1=ii-dis
            i2=ii+dis
            ct=0
            sw=0.0
            sf=0.0
            do j=j1, j2
              do i=i1, i2
                dis2=(j-jj)*(j-jj)+(i-ii)*(i-ii)
                if( dis2.le.dis2max ) then
                  js=j
                  if( i.lt.1 ) then
                    is=i+NX
                  elseif( i.ge.1.and.i.le.NX ) then
                    is=i
                  else
                    is=i-NX
                  endif
                  if( fct(is,js).ge.0.0 ) then
                    wt=1.0/(sqrt(float(dis2))+1.0)
                    ct=ct+1
                    sw=sw+wt
                    sf=sf+wt*fct(is,js)
                  endif
                endif
              enddo
            enddo
            if( ct.ge.10.and.sw.gt.0.0 ) then
              fct0(ii,jj)=sf/sw
            else
              dis=dis+1
              goto 100
            endif
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c output 0.25deg-PEN adjust factors over ocean
c-----------------------------------------------------------------------
      open(20, file=f_025, access='direct', recl=NX*NY*4)
      write(20, rec=1) fct0
      close(20)
      write(*, *) '  ==> ', f_025

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
