c@***h* CMORPH-CDR/o6.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-DLY.f
c
c=======================================================================
c PROGRAM
c   o6.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-DLY.f
c
c LANGUAGE
c   Fortran77
c
c-----------------------------------------------------------------------
c PURPOSE
c   to generate 0.25deg-DLY adjust factors over ocean using 0.25deg-PEN
c   adjust factors over ocean for a 3-pentad processing window
c
c-----------------------------------------------------------------------
c USAGE
c   o6.CMORPH_V1.0_CRT.get_FCT_OCN_0.25deg-DLY.x <yyyymmdd> [<redo>]
c
c-----------------------------------------------------------------------
c INPUT FILES
c   CMORPH_V1.0_LAND_MASK_0.25deg.lnx : 0.25deg land mask
c   FCT_OCN_0.25deg-PEN_<yyyypp>      : 0.25deg-PEN adjust factors over
c                                       ocean
c
c OUTPUT FILES
c   FCT_OCN_0.25deg-DLY_<yyyymmdd> : 0.25deg-DLY adjust factors over ocean
c
c-----------------------------------------------------------------------
c INPUT VARIABLES
c   yyyymmdd : target date
c   redo     : optional flag for re-processing
c
c LOCAL VARIABLES
c   NX         : X-dimension (0-360) in 0.25deg grid
c   NY         : Y-dimension (60S-60N) in 0.25deg grid
c   NP         : number of pentad in a year
c   WIN_P      : number of pentad in processing window
c   WIN_D      : number of days in processing window
c   HW_D       : number of days in half of smoothing window
c   -----------:-------------------------------------------
c   yyyymmdd   : target date
c   f_msk      : 0.25deg land mask file name
c   f_pen      : file name of 0.25deg-PEN adjust factors over ocean
c   f_dly      : file name of 0.25deg-DLY adjust factors over ocean
c   yyyy,mm,dd : target date
c   days()     : number of days in each month
c   leap       : leap-year flag
c   jjj        : Julian day (non-leap-year)
c   m          : loop counter for months
c   m5         : mod(jjj,5)
c   pp         : target pentad
c   ct         : count of valid data points
c   dis        : distance between working and target days
c   year,pent  : working pentad
c   pday()     : number of days in each pentad over processing window
c   dc         : count of days
c   k          : loop counter for pentad
c   d          : loop counter for days
c   td         : index for target date
c   d1,d2      : boundaries for smoothing window
c   ii         : loop counter in X-direction
c   jj         : loop counter in Y-direction
c   mask()     : 0.25deg land mask
c   fct_p()    : 0.25deg-PEN adjust factors over ocean
c   fct_d()    : 0.25deg-DLY adjust factors over ocean
c   fct_w()    : working arry for 0.25deg-DLY adjust factors
c   wt         : weight
c   sw         : sum of weights over smoothing window
c   sf         : sum of adjust factors over smoothing window
c   pen_there  : check if 0.25deg-PEN data file exists
c
c-----------------------------------------------------------------------
c NOTE
c   modified from program 'def_adj_factor_0.25deg_dly_ocean.lnx.f'
c
c   step 6 of 6 to generate 0.25deg-DLY adjust factors over ocean
c
c@*****
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program o6_CMORPH_V10_CRT_get_FCT_OCN_025deg_DLY
      implicit none

      integer NX, NY, NP, WIN_P, WIN_D, HW_D
      parameter( NX=1440, NY=480, NP=73 )
      parameter( WIN_P=3, WIN_D=1+5*WIN_P, HW_D=2 )

      character yyyymmdd*8, f_msk*42, f_pen*34, f_dly*36
      integer yyyy, mm, dd, days(12), leap, jjj, m, m5, pp, ct, dis
      integer year, pent, pday(WIN_P), dc, k, d, td, d1, d2, ii, jj
      real mask(NX,NY), fct_p(NX,NY), fct_d(NX,NY), fct_w(NX,NY,WIN_D)
      real wt, sw, sf
      logical pen_there

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c get target date
c-----------------------------------------------------------------------
      if( iargc().lt.1 ) then
        write(*, *)
        write(*, *) '##########################################'
        write(*, *) '#                                        #'
        write(*, *) '#  parameter <yyyymmdd> [<redo>] needed  #'
        write(*, *) '#                                        #'
        write(*, *) '##########################################'
        write(*, *)
        stop
      endif
      call getarg(1, yyyymmdd)
      read(yyyymmdd, '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c check for leap-year
c-----------------------------------------------------------------------
      leap=0
      if( mod(yyyy,4).eq.0 )   leap=1
      if( mod(yyyy,100).eq.0 ) leap=0
      if( mod(yyyy,400).eq.0 ) leap=1

c-----------------------------------------------------------------------
c get Julian day
c-----------------------------------------------------------------------
      jjj=dd
      do m=1, mm-1
        jjj=jjj+days(m)
      enddo
      if( mm.eq.2.and.dd.eq.29 ) jjj=59

c-----------------------------------------------------------------------
c get target pentad
c-----------------------------------------------------------------------
      m5=mod(jjj,5)
      if( mm.eq.2.and.dd.eq.29 ) m5=5
      pp=jjj/5
      if( m5.ne.0 ) pp=pp+1

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|12<--
      f_msk='../table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_pen='../work/FCT_OCN_0.25deg-PEN_yyyypp'
      f_dly='../work/FCT_OCN_0.25deg-DLY_yyyymmdd'

      if( iargc().ge.2 ) then
        f_pen(4:7)='redo'
        f_dly(4:7)='redo'
      endif

      write(f_dly(29:36), '(i4.4,2i2.2)') yyyy, mm, dd

c-----------------------------------------------------------------------
c read in 0.25deg land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old',
     1    access='direct', recl=NX*NY*4)
      write(*, *) ' <--  ', f_msk
      read(10, rec=1) mask
      close(10)

c-----------------------------------------------------------------------
c read in 0.25deg-PEN adjust factors over 3-pentad processing window
c-----------------------------------------------------------------------
      dc=0
      do k=1, WIN_P

        pent=pp-2+k
        year=yyyy
        if( pent.lt.1 ) then
          pent=pent+NP
          year=yyyy-1
        else if( pent.gt.NP ) then
          pent=pent-NP
          year=yyyy+1
        endif

        pday(k)=5
        if( pent.eq.12 ) pday(k)=5+leap

        write(f_pen(29:34), '(i4.4,i2.2)') year, pent

        inquire(file=f_pen, exist=pen_there)
        if( .not.pen_there ) then
          write(*, *) '#### file missing: ', f_pen
          stop
        endif

        open(11, file=f_pen, status='old',
     1      access='direct', recl=NX*NY*4)
        write(*, *) ' <--  ', f_pen
        read(11, rec=1) fct_p
        close(11)

        do d=1, pday(k)
          dc=dc+1
          do jj=1, NY
            do ii=1, NX
              fct_w(ii,jj,dc)=fct_p(ii,jj)
            enddo
          enddo
        enddo

      enddo

c-----------------------------------------------------------------------
c perform temporal smoothing over a 5-day smoothing window
c-----------------------------------------------------------------------
      if( m5.eq.0 ) then
        td=pday(1)+pday(2)
      else
        td=pday(1)+m5
      endif
      d1=td-HW_D
      d2=td+HW_D
      do jj=1, NY
        do ii=1, NX
          if( mask(ii,jj).ne.0.0 ) then
            fct_d(ii,jj)=-999.0
          else
            ct=0
            sw=0.0
            sf=0.0
            do d=d1, d2
              if( fct_w(ii,jj,d).ge.0.0 ) then
                dis=abs(d-td)
                wt=1.0/(dis+1.0)
                ct=ct+1
                sw=sw+wt
                sf=sf+wt*fct_w(ii,jj,d)
              endif
            enddo
            if( ct.gt.0.and.sw.gt.0.0 ) then
              fct_d(ii,jj)=sf/sw
            else
              fct_d(ii,jj)=-999.0
            endif
          endif
        enddo
      enddo

c-----------------------------------------------------------------------
c output 0.25deg-DLY adjust factors over ocean
c-----------------------------------------------------------------------
      open(20, file=f_dly, access='direct', recl=NX*NY*4)
      write(20, rec=1) fct_d
      close(20)
      write(*, *) '  ==> ', f_dly

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
